# -*- coding: utf-8 -*-

# File generated according to DMatSetup.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from .....GUI.Tools.FloatEdit import FloatEdit
from .....GUI.Tools.WImport.WImport import WImport

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_DMatSetup(object):
    def setupUi(self, DMatSetup):
        if not DMatSetup.objectName():
            DMatSetup.setObjectName(u"DMatSetup")
        DMatSetup.resize(642, 451)
        self.verticalLayout = QVBoxLayout(DMatSetup)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.in_name = QLabel(DMatSetup)
        self.in_name.setObjectName(u"in_name")

        self.horizontalLayout.addWidget(self.in_name)

        self.le_name = QLineEdit(DMatSetup)
        self.le_name.setObjectName(u"le_name")

        self.horizontalLayout.addWidget(self.le_name)

        self.verticalLayout.addLayout(self.horizontalLayout)

        self.is_isotropic = QCheckBox(DMatSetup)
        self.is_isotropic.setObjectName(u"is_isotropic")

        self.verticalLayout.addWidget(self.is_isotropic)

        self.nav_phy = QTabWidget(DMatSetup)
        self.nav_phy.setObjectName(u"nav_phy")
        self.nav_phy.setMinimumSize(QSize(370, 0))
        self.tab_elec = QWidget()
        self.tab_elec.setObjectName(u"tab_elec")
        self.gridLayout_2 = QGridLayout(self.tab_elec)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.lf_rho_elec = FloatEdit(self.tab_elec)
        self.lf_rho_elec.setObjectName(u"lf_rho_elec")

        self.gridLayout_2.addWidget(self.lf_rho_elec, 0, 1, 1, 1)

        self.lf_epsr = FloatEdit(self.tab_elec)
        self.lf_epsr.setObjectName(u"lf_epsr")

        self.gridLayout_2.addWidget(self.lf_epsr, 1, 1, 1, 1)

        self.unit_rho_elec = QLabel(self.tab_elec)
        self.unit_rho_elec.setObjectName(u"unit_rho_elec")
        font = QFont()
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.unit_rho_elec.setFont(font)

        self.gridLayout_2.addWidget(self.unit_rho_elec, 0, 2, 1, 1)

        self.in_epsr = QLabel(self.tab_elec)
        self.in_epsr.setObjectName(u"in_epsr")
        self.in_epsr.setFont(font)

        self.gridLayout_2.addWidget(self.in_epsr, 1, 0, 1, 1)

        self.in_rho_elec = QLabel(self.tab_elec)
        self.in_rho_elec.setObjectName(u"in_rho_elec")
        self.in_rho_elec.setFont(font)

        self.gridLayout_2.addWidget(self.in_rho_elec, 0, 0, 1, 1)

        self.verticalSpacer_2 = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.gridLayout_2.addItem(self.verticalSpacer_2, 2, 1, 1, 1)

        self.nav_phy.addTab(self.tab_elec, "")
        self.tab_mag = QWidget()
        self.tab_mag.setObjectName(u"tab_mag")
        self.verticalLayout_3 = QVBoxLayout(self.tab_mag)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.in_mur_lin = QLabel(self.tab_mag)
        self.in_mur_lin.setObjectName(u"in_mur_lin")
        self.in_mur_lin.setFont(font)

        self.gridLayout.addWidget(self.in_mur_lin, 0, 0, 1, 1)

        self.lf_mur_lin = FloatEdit(self.tab_mag)
        self.lf_mur_lin.setObjectName(u"lf_mur_lin")

        self.gridLayout.addWidget(self.lf_mur_lin, 0, 1, 1, 1)

        self.in_Brm20 = QLabel(self.tab_mag)
        self.in_Brm20.setObjectName(u"in_Brm20")
        self.in_Brm20.setFont(font)

        self.gridLayout.addWidget(self.in_Brm20, 1, 0, 1, 1)

        self.lf_Brm20 = FloatEdit(self.tab_mag)
        self.lf_Brm20.setObjectName(u"lf_Brm20")

        self.gridLayout.addWidget(self.lf_Brm20, 1, 1, 1, 1)

        self.unit_Brm20 = QLabel(self.tab_mag)
        self.unit_Brm20.setObjectName(u"unit_Brm20")
        self.unit_Brm20.setFont(font)

        self.gridLayout.addWidget(self.unit_Brm20, 1, 2, 1, 1)

        self.in_alpha_Br = QLabel(self.tab_mag)
        self.in_alpha_Br.setObjectName(u"in_alpha_Br")
        self.in_alpha_Br.setFont(font)

        self.gridLayout.addWidget(self.in_alpha_Br, 2, 0, 1, 1)

        self.lf_alpha_Br = FloatEdit(self.tab_mag)
        self.lf_alpha_Br.setObjectName(u"lf_alpha_Br")

        self.gridLayout.addWidget(self.lf_alpha_Br, 2, 1, 1, 1)

        self.in_Wlam = QLabel(self.tab_mag)
        self.in_Wlam.setObjectName(u"in_Wlam")
        self.in_Wlam.setFont(font)

        self.gridLayout.addWidget(self.in_Wlam, 3, 0, 1, 1)

        self.lf_Wlam = FloatEdit(self.tab_mag)
        self.lf_Wlam.setObjectName(u"lf_Wlam")

        self.gridLayout.addWidget(self.lf_Wlam, 3, 1, 1, 1)

        self.unit_Wlam = QLabel(self.tab_mag)
        self.unit_Wlam.setObjectName(u"unit_Wlam")
        self.unit_Wlam.setFont(font)

        self.gridLayout.addWidget(self.unit_Wlam, 3, 2, 1, 1)

        self.verticalLayout_3.addLayout(self.gridLayout)

        self.w_BH_import = WImport(self.tab_mag)
        self.w_BH_import.setObjectName(u"w_BH_import")

        self.verticalLayout_3.addWidget(self.w_BH_import)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_3.addItem(self.verticalSpacer)

        self.nav_phy.addTab(self.tab_mag, "")
        self.tab_mec = QWidget()
        self.tab_mec.setObjectName(u"tab_mec")
        self.verticalLayout_12 = QVBoxLayout(self.tab_mec)
        self.verticalLayout_12.setObjectName(u"verticalLayout_12")
        self.horizontalLayout_41 = QHBoxLayout()
        self.horizontalLayout_41.setObjectName(u"horizontalLayout_41")
        self.in_rho_meca = QLabel(self.tab_mec)
        self.in_rho_meca.setObjectName(u"in_rho_meca")
        self.in_rho_meca.setFont(font)

        self.horizontalLayout_41.addWidget(self.in_rho_meca)

        self.lf_rho_meca = FloatEdit(self.tab_mec)
        self.lf_rho_meca.setObjectName(u"lf_rho_meca")

        self.horizontalLayout_41.addWidget(self.lf_rho_meca)

        self.unit_rho_meca = QLabel(self.tab_mec)
        self.unit_rho_meca.setObjectName(u"unit_rho_meca")
        self.unit_rho_meca.setFont(font)

        self.horizontalLayout_41.addWidget(self.unit_rho_meca)

        self.verticalLayout_12.addLayout(self.horizontalLayout_41)

        self.nav_meca = QStackedWidget(self.tab_mec)
        self.nav_meca.setObjectName(u"nav_meca")
        self.page_niso_mec = QWidget()
        self.page_niso_mec.setObjectName(u"page_niso_mec")
        self.verticalLayout_4 = QVBoxLayout(self.page_niso_mec)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.g_young = QGroupBox(self.page_niso_mec)
        self.g_young.setObjectName(u"g_young")
        self.horizontalLayout_2 = QHBoxLayout(self.g_young)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.in_Ex = QLabel(self.g_young)
        self.in_Ex.setObjectName(u"in_Ex")
        self.in_Ex.setFont(font)

        self.horizontalLayout_2.addWidget(self.in_Ex)

        self.lf_Ex = FloatEdit(self.g_young)
        self.lf_Ex.setObjectName(u"lf_Ex")

        self.horizontalLayout_2.addWidget(self.lf_Ex)

        self.in_Ey = QLabel(self.g_young)
        self.in_Ey.setObjectName(u"in_Ey")
        self.in_Ey.setFont(font)

        self.horizontalLayout_2.addWidget(self.in_Ey)

        self.lf_Ey = FloatEdit(self.g_young)
        self.lf_Ey.setObjectName(u"lf_Ey")

        self.horizontalLayout_2.addWidget(self.lf_Ey)

        self.in_Ez = QLabel(self.g_young)
        self.in_Ez.setObjectName(u"in_Ez")
        self.in_Ez.setFont(font)

        self.horizontalLayout_2.addWidget(self.in_Ez)

        self.lf_Ez = FloatEdit(self.g_young)
        self.lf_Ez.setObjectName(u"lf_Ez")

        self.horizontalLayout_2.addWidget(self.lf_Ez)

        self.verticalLayout_4.addWidget(self.g_young)

        self.g_poisson = QGroupBox(self.page_niso_mec)
        self.g_poisson.setObjectName(u"g_poisson")
        self.horizontalLayout_3 = QHBoxLayout(self.g_poisson)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.in_nu_xy = QLabel(self.g_poisson)
        self.in_nu_xy.setObjectName(u"in_nu_xy")
        self.in_nu_xy.setFont(font)

        self.horizontalLayout_3.addWidget(self.in_nu_xy)

        self.lf_nu_xy = FloatEdit(self.g_poisson)
        self.lf_nu_xy.setObjectName(u"lf_nu_xy")

        self.horizontalLayout_3.addWidget(self.lf_nu_xy)

        self.in_nu_xz = QLabel(self.g_poisson)
        self.in_nu_xz.setObjectName(u"in_nu_xz")
        self.in_nu_xz.setFont(font)

        self.horizontalLayout_3.addWidget(self.in_nu_xz)

        self.lf_nu_xz = FloatEdit(self.g_poisson)
        self.lf_nu_xz.setObjectName(u"lf_nu_xz")

        self.horizontalLayout_3.addWidget(self.lf_nu_xz)

        self.in_nu_yz = QLabel(self.g_poisson)
        self.in_nu_yz.setObjectName(u"in_nu_yz")
        self.in_nu_yz.setFont(font)

        self.horizontalLayout_3.addWidget(self.in_nu_yz)

        self.lf_nu_yz = FloatEdit(self.g_poisson)
        self.lf_nu_yz.setObjectName(u"lf_nu_yz")

        self.horizontalLayout_3.addWidget(self.lf_nu_yz)

        self.verticalLayout_4.addWidget(self.g_poisson)

        self.g_shear = QGroupBox(self.page_niso_mec)
        self.g_shear.setObjectName(u"g_shear")
        self.horizontalLayout_4 = QHBoxLayout(self.g_shear)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.in_Gxy = QLabel(self.g_shear)
        self.in_Gxy.setObjectName(u"in_Gxy")
        self.in_Gxy.setFont(font)

        self.horizontalLayout_4.addWidget(self.in_Gxy)

        self.lf_Gxy = FloatEdit(self.g_shear)
        self.lf_Gxy.setObjectName(u"lf_Gxy")

        self.horizontalLayout_4.addWidget(self.lf_Gxy)

        self.in_Gxz = QLabel(self.g_shear)
        self.in_Gxz.setObjectName(u"in_Gxz")
        self.in_Gxz.setFont(font)

        self.horizontalLayout_4.addWidget(self.in_Gxz)

        self.lf_Gxz = FloatEdit(self.g_shear)
        self.lf_Gxz.setObjectName(u"lf_Gxz")

        self.horizontalLayout_4.addWidget(self.lf_Gxz)

        self.in_Gyz = QLabel(self.g_shear)
        self.in_Gyz.setObjectName(u"in_Gyz")
        self.in_Gyz.setFont(font)

        self.horizontalLayout_4.addWidget(self.in_Gyz)

        self.lf_Gyz = FloatEdit(self.g_shear)
        self.lf_Gyz.setObjectName(u"lf_Gyz")

        self.horizontalLayout_4.addWidget(self.lf_Gyz)

        self.verticalLayout_4.addWidget(self.g_shear)

        self.verticalSpacer_5 = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_4.addItem(self.verticalSpacer_5)

        self.nav_meca.addWidget(self.page_niso_mec)
        self.page_iso_mec = QWidget()
        self.page_iso_mec.setObjectName(u"page_iso_mec")
        self.gridLayout_6 = QGridLayout(self.page_iso_mec)
        self.gridLayout_6.setObjectName(u"gridLayout_6")
        self.in_E = QLabel(self.page_iso_mec)
        self.in_E.setObjectName(u"in_E")
        self.in_E.setFont(font)

        self.gridLayout_6.addWidget(self.in_E, 0, 0, 1, 1)

        self.lf_G = FloatEdit(self.page_iso_mec)
        self.lf_G.setObjectName(u"lf_G")

        self.gridLayout_6.addWidget(self.lf_G, 2, 2, 1, 2)

        self.verticalSpacer_6 = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.gridLayout_6.addItem(self.verticalSpacer_6, 4, 3, 1, 1)

        self.unit_G = QLabel(self.page_iso_mec)
        self.unit_G.setObjectName(u"unit_G")

        self.gridLayout_6.addWidget(self.unit_G, 2, 4, 1, 1)

        self.lf_E = FloatEdit(self.page_iso_mec)
        self.lf_E.setObjectName(u"lf_E")

        self.gridLayout_6.addWidget(self.lf_E, 0, 1, 1, 3)

        self.in_G = QLabel(self.page_iso_mec)
        self.in_G.setObjectName(u"in_G")
        self.in_G.setFont(font)

        self.gridLayout_6.addWidget(self.in_G, 2, 0, 1, 2)

        self.in_nu = QLabel(self.page_iso_mec)
        self.in_nu.setObjectName(u"in_nu")
        self.in_nu.setFont(font)

        self.gridLayout_6.addWidget(self.in_nu, 1, 0, 1, 3)

        self.lf_nu = FloatEdit(self.page_iso_mec)
        self.lf_nu.setObjectName(u"lf_nu")

        self.gridLayout_6.addWidget(self.lf_nu, 1, 3, 1, 1)

        self.unit_E = QLabel(self.page_iso_mec)
        self.unit_E.setObjectName(u"unit_E")

        self.gridLayout_6.addWidget(self.unit_E, 0, 4, 1, 1)

        self.nav_meca.addWidget(self.page_iso_mec)

        self.verticalLayout_12.addWidget(self.nav_meca)

        self.nav_phy.addTab(self.tab_mec, "")
        self.tab_ther = QWidget()
        self.tab_ther.setObjectName(u"tab_ther")
        self.verticalLayout_2 = QVBoxLayout(self.tab_ther)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.g_lambda = QGroupBox(self.tab_ther)
        self.g_lambda.setObjectName(u"g_lambda")
        self.g_lambda.setMaximumSize(QSize(16777215, 80))
        self.verticalLayout_11 = QVBoxLayout(self.g_lambda)
        self.verticalLayout_11.setObjectName(u"verticalLayout_11")
        self.nav_ther = QStackedWidget(self.g_lambda)
        self.nav_ther.setObjectName(u"nav_ther")
        self.page_niso_ther = QWidget()
        self.page_niso_ther.setObjectName(u"page_niso_ther")
        self.horizontalLayout_5 = QHBoxLayout(self.page_niso_ther)
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.in_Lx = QLabel(self.page_niso_ther)
        self.in_Lx.setObjectName(u"in_Lx")
        self.in_Lx.setFont(font)

        self.horizontalLayout_5.addWidget(self.in_Lx)

        self.lf_Lx = FloatEdit(self.page_niso_ther)
        self.lf_Lx.setObjectName(u"lf_Lx")

        self.horizontalLayout_5.addWidget(self.lf_Lx)

        self.in_Ly = QLabel(self.page_niso_ther)
        self.in_Ly.setObjectName(u"in_Ly")
        self.in_Ly.setFont(font)

        self.horizontalLayout_5.addWidget(self.in_Ly)

        self.lf_Ly = FloatEdit(self.page_niso_ther)
        self.lf_Ly.setObjectName(u"lf_Ly")

        self.horizontalLayout_5.addWidget(self.lf_Ly)

        self.in_Lz = QLabel(self.page_niso_ther)
        self.in_Lz.setObjectName(u"in_Lz")
        self.in_Lz.setFont(font)

        self.horizontalLayout_5.addWidget(self.in_Lz)

        self.lf_Lz = FloatEdit(self.page_niso_ther)
        self.lf_Lz.setObjectName(u"lf_Lz")

        self.horizontalLayout_5.addWidget(self.lf_Lz)

        self.nav_ther.addWidget(self.page_niso_ther)
        self.page_iso_ther = QWidget()
        self.page_iso_ther.setObjectName(u"page_iso_ther")
        self.horizontalLayout_7 = QHBoxLayout(self.page_iso_ther)
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.in_L = QLabel(self.page_iso_ther)
        self.in_L.setObjectName(u"in_L")
        self.in_L.setFont(font)

        self.horizontalLayout_7.addWidget(self.in_L)

        self.lf_L = FloatEdit(self.page_iso_ther)
        self.lf_L.setObjectName(u"lf_L")

        self.horizontalLayout_7.addWidget(self.lf_L)

        self.unit_L = QLabel(self.page_iso_ther)
        self.unit_L.setObjectName(u"unit_L")

        self.horizontalLayout_7.addWidget(self.unit_L)

        self.nav_ther.addWidget(self.page_iso_ther)

        self.verticalLayout_11.addWidget(self.nav_ther)

        self.verticalLayout_2.addWidget(self.g_lambda)

        self.gridLayout_3 = QGridLayout()
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.in_alpha = QLabel(self.tab_ther)
        self.in_alpha.setObjectName(u"in_alpha")
        self.in_alpha.setFont(font)

        self.gridLayout_3.addWidget(self.in_alpha, 1, 0, 1, 1)

        self.in_Cp = QLabel(self.tab_ther)
        self.in_Cp.setObjectName(u"in_Cp")
        self.in_Cp.setFont(font)

        self.gridLayout_3.addWidget(self.in_Cp, 0, 0, 1, 1)

        self.lf_alpha = FloatEdit(self.tab_ther)
        self.lf_alpha.setObjectName(u"lf_alpha")

        self.gridLayout_3.addWidget(self.lf_alpha, 1, 1, 1, 1)

        self.unit_Cp = QLabel(self.tab_ther)
        self.unit_Cp.setObjectName(u"unit_Cp")
        self.unit_Cp.setFont(font)

        self.gridLayout_3.addWidget(self.unit_Cp, 0, 2, 1, 1)

        self.lf_Cp = FloatEdit(self.tab_ther)
        self.lf_Cp.setObjectName(u"lf_Cp")

        self.gridLayout_3.addWidget(self.lf_Cp, 0, 1, 1, 1)

        self.verticalLayout_2.addLayout(self.gridLayout_3)

        self.verticalSpacer_3 = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_2.addItem(self.verticalSpacer_3)

        self.nav_phy.addTab(self.tab_ther, "")
        self.tab_eco = QWidget()
        self.tab_eco.setObjectName(u"tab_eco")
        self.verticalLayout_6 = QVBoxLayout(self.tab_eco)
        self.verticalLayout_6.setObjectName(u"verticalLayout_6")
        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.in_cost_unit = QLabel(self.tab_eco)
        self.in_cost_unit.setObjectName(u"in_cost_unit")
        self.in_cost_unit.setFont(font)

        self.horizontalLayout_6.addWidget(self.in_cost_unit)

        self.lf_cost_unit = FloatEdit(self.tab_eco)
        self.lf_cost_unit.setObjectName(u"lf_cost_unit")

        self.horizontalLayout_6.addWidget(self.lf_cost_unit)

        self.unit_cost_unit = QLabel(self.tab_eco)
        self.unit_cost_unit.setObjectName(u"unit_cost_unit")

        self.horizontalLayout_6.addWidget(self.unit_cost_unit)

        self.verticalLayout_6.addLayout(self.horizontalLayout_6)

        self.verticalSpacer_4 = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_6.addItem(self.verticalSpacer_4)

        self.nav_phy.addTab(self.tab_eco, "")

        self.verticalLayout.addWidget(self.nav_phy)

        self.horizontalGroupBox = QGroupBox(DMatSetup)
        self.horizontalGroupBox.setObjectName(u"horizontalGroupBox")
        self.horizontalLayout_9 = QHBoxLayout(self.horizontalGroupBox)
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.horizontalSpacer_2 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_9.addItem(self.horizontalSpacer_2)

        self.b_add_matlib = QPushButton(self.horizontalGroupBox)
        self.b_add_matlib.setObjectName(u"b_add_matlib")

        self.horizontalLayout_9.addWidget(self.b_add_matlib)

        self.b_save = QPushButton(self.horizontalGroupBox)
        self.b_save.setObjectName(u"b_save")

        self.horizontalLayout_9.addWidget(self.b_save)

        self.b_cancel = QPushButton(self.horizontalGroupBox)
        self.b_cancel.setObjectName(u"b_cancel")

        self.horizontalLayout_9.addWidget(self.b_cancel)

        self.verticalLayout.addWidget(self.horizontalGroupBox)

        self.retranslateUi(DMatSetup)

        self.nav_phy.setCurrentIndex(1)
        self.nav_meca.setCurrentIndex(1)
        self.nav_ther.setCurrentIndex(1)

        QMetaObject.connectSlotsByName(DMatSetup)

    # setupUi

    def retranslateUi(self, DMatSetup):
        DMatSetup.setWindowTitle(
            QCoreApplication.translate("DMatSetup", u"Edit Material", None)
        )
        self.in_name.setText(QCoreApplication.translate("DMatSetup", u"name", None))
        self.le_name.setText("")
        self.is_isotropic.setText(
            QCoreApplication.translate("DMatSetup", u"is_isotropic", None)
        )
        self.unit_rho_elec.setText(
            QCoreApplication.translate("DMatSetup", u"ohm.m", None)
        )
        self.in_epsr.setText(QCoreApplication.translate("DMatSetup", u"epsr", None))
        self.in_rho_elec.setText(QCoreApplication.translate("DMatSetup", u"rho", None))
        self.nav_phy.setTabText(
            self.nav_phy.indexOf(self.tab_elec),
            QCoreApplication.translate("DMatSetup", u"Electrical", None),
        )
        self.in_mur_lin.setText(
            QCoreApplication.translate("DMatSetup", u"mur_lin", None)
        )
        self.in_Brm20.setText(QCoreApplication.translate("DMatSetup", u"Brm20", None))
        self.unit_Brm20.setText(QCoreApplication.translate("DMatSetup", u"T", None))
        self.in_alpha_Br.setText(
            QCoreApplication.translate("DMatSetup", u"alphaBr", None)
        )
        self.in_Wlam.setText(QCoreApplication.translate("DMatSetup", u"Wlam", None))
        self.unit_Wlam.setText(QCoreApplication.translate("DMatSetup", u"m", None))
        self.nav_phy.setTabText(
            self.nav_phy.indexOf(self.tab_mag),
            QCoreApplication.translate("DMatSetup", u"Magnetics", None),
        )
        self.in_rho_meca.setText(QCoreApplication.translate("DMatSetup", u"rho", None))
        self.unit_rho_meca.setText(
            QCoreApplication.translate("DMatSetup", u"kg/m^3", None)
        )
        self.g_young.setTitle(
            QCoreApplication.translate(
                "DMatSetup", u"Equivalent Yong Modulus [Pa]", None
            )
        )
        self.in_Ex.setText(QCoreApplication.translate("DMatSetup", u"Ex", None))
        self.in_Ey.setText(QCoreApplication.translate("DMatSetup", u"Ey", None))
        self.in_Ez.setText(QCoreApplication.translate("DMatSetup", u"Ez", None))
        self.g_poisson.setTitle(
            QCoreApplication.translate(
                "DMatSetup", u"Equivalent Poisson ratio [ ]", None
            )
        )
        self.in_nu_xy.setText(QCoreApplication.translate("DMatSetup", u"nu_xy", None))
        self.in_nu_xz.setText(QCoreApplication.translate("DMatSetup", u"nu_xz", None))
        self.in_nu_yz.setText(QCoreApplication.translate("DMatSetup", u"nu_yz", None))
        self.g_shear.setTitle(
            QCoreApplication.translate("DMatSetup", u"Shear modulus [Pa]", None)
        )
        self.in_Gxy.setText(QCoreApplication.translate("DMatSetup", u"Gxy", None))
        self.in_Gxz.setText(QCoreApplication.translate("DMatSetup", u"Gxz", None))
        self.in_Gyz.setText(QCoreApplication.translate("DMatSetup", u"Gyz", None))
        self.in_E.setText(QCoreApplication.translate("DMatSetup", u"E", None))
        self.unit_G.setText(QCoreApplication.translate("DMatSetup", u"Pa", None))
        self.in_G.setText(QCoreApplication.translate("DMatSetup", u"G", None))
        self.in_nu.setText(QCoreApplication.translate("DMatSetup", u"nu", None))
        self.unit_E.setText(QCoreApplication.translate("DMatSetup", u"Pa", None))
        self.nav_phy.setTabText(
            self.nav_phy.indexOf(self.tab_mec),
            QCoreApplication.translate("DMatSetup", u"Mechanics", None),
        )
        self.g_lambda.setTitle(
            QCoreApplication.translate("DMatSetup", u"Lambda [W/K]", None)
        )
        self.in_Lx.setText(QCoreApplication.translate("DMatSetup", u"X", None))
        self.in_Ly.setText(QCoreApplication.translate("DMatSetup", u"Y", None))
        self.in_Lz.setText(QCoreApplication.translate("DMatSetup", u"Z", None))
        self.in_L.setText(QCoreApplication.translate("DMatSetup", u"Lambda", None))
        self.unit_L.setText(QCoreApplication.translate("DMatSetup", u"W / K", None))
        self.in_alpha.setText(QCoreApplication.translate("DMatSetup", u"alpha", None))
        self.in_Cp.setText(QCoreApplication.translate("DMatSetup", u"Cp", None))
        self.unit_Cp.setText(
            QCoreApplication.translate("DMatSetup", u"W / kg / K", None)
        )
        self.nav_phy.setTabText(
            self.nav_phy.indexOf(self.tab_ther),
            QCoreApplication.translate("DMatSetup", u"Thermics", None),
        )
        self.in_cost_unit.setText(
            QCoreApplication.translate("DMatSetup", u"cost_unit", None)
        )
        self.unit_cost_unit.setText(
            QCoreApplication.translate("DMatSetup", u"\u20ac / kg", None)
        )
        self.nav_phy.setTabText(
            self.nav_phy.indexOf(self.tab_eco),
            QCoreApplication.translate("DMatSetup", u"Economical", None),
        )
        self.b_add_matlib.setText(
            QCoreApplication.translate("DMatSetup", u"Add to MatLib", None)
        )
        self.b_save.setText(QCoreApplication.translate("DMatSetup", u"Save", None))
        self.b_cancel.setText(QCoreApplication.translate("DMatSetup", u"Cancel", None))

    # retranslateUi



# -*- coding: utf-8 -*-


def comp_length(self):
    """Compute the total length of the Lamination (including radial
    ventilations duct)

    Parameters
    ----------
    self : Lamination
        A Lamination object

    Returns
    -------
    Lt: float
        Total Lenght of the Lamination [m]

    """
    if self.Nrvd is None or self.Wrvd is None:
        return self.L1
    else:
        return self.L1 + self.Nrvd * self.Wrvd



# -*- coding: utf-8 -*-

from scipy.io import loadmat
from ....Methods.Import.ImportMatlab import MatFileError
from os.path import isfile
from numpy import squeeze


def get_data(self):
    """Return the value from the .mat file

    Parameters
    ----------
    self : ImportMatlab
        An ImportMatlab object

    Returns
    -------
    value : ?
        Loaded data
    """

    if self.file_path[-4:] != ".mat":
        self.file_path += ".mat"
    if not isfile(self.file_path):
        raise MatFileError("ERROR: The mat file doesn't exist " + self.file_path)

    data_dict = loadmat(self.file_path)
    if self.var_name not in data_dict:
        raise MatFileError(
            "ERROR: The variable "
            + self.var_name
            + " is not available in the file "
            + self.file_path
        )

    return squeeze(data_dict[self.var_name])



# -*- coding: utf-8 -*-
from ....Functions.Geometry.inter_line_line import find_line_eq
from numpy import sqrt, angle, exp, sign


def comp_distance(self, Z):
    """Compute the distance of a point to the Arc

    Parameters
    ----------
    self : Arc
        An Arc object
    Z : complex
        Complex coordinate of the point

    Returns
    -------
    D : float
        distance of a point to the Segment
    """

    Zc = self.get_center()
    Z2 = Z - Zc

    A1 = self.get_angle()
    A2 = angle(Z2 * exp(-1j * angle(self.get_begin() - Zc)))

    if sign(A1) == sign(A2) and abs(A2) < abs(A1):
        # The point is on the correct side of the arc
        D1 = abs(sqrt(Z2.real ** 2 + Z2.imag ** 2) - self.comp_radius())
        return D1
    else:
        D2 = abs(Z - self.get_begin())
        D3 = abs(Z - self.get_end())
        D4 = abs(Z - self.get_middle())
        return min([D2, D3, D4])



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/CondType11.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/CondType11
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Conductor import Conductor

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.CondType11.comp_surface_active import comp_surface_active
except ImportError as error:
    comp_surface_active = error

try:
    from ..Methods.Machine.CondType11.comp_height import comp_height
except ImportError as error:
    comp_height = error

try:
    from ..Methods.Machine.CondType11.comp_surface import comp_surface
except ImportError as error:
    comp_surface = error

try:
    from ..Methods.Machine.CondType11.comp_width import comp_width
except ImportError as error:
    comp_width = error

try:
    from ..Methods.Machine.CondType11.plot import plot
except ImportError as error:
    plot = error

try:
    from ..Methods.Machine.CondType11.plot_schematics import plot_schematics
except ImportError as error:
    plot_schematics = error


from ._check import InitUnKnowClassError
from .Material import Material


class CondType11(Conductor):
    """parallel stranded conductor consisting of at least a single rectangular wire"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Machine.CondType11.comp_surface_active
    if isinstance(comp_surface_active, ImportError):
        comp_surface_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use CondType11 method comp_surface_active: "
                    + str(comp_surface_active)
                )
            )
        )
    else:
        comp_surface_active = comp_surface_active
    # cf Methods.Machine.CondType11.comp_height
    if isinstance(comp_height, ImportError):
        comp_height = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use CondType11 method comp_height: " + str(comp_height)
                )
            )
        )
    else:
        comp_height = comp_height
    # cf Methods.Machine.CondType11.comp_surface
    if isinstance(comp_surface, ImportError):
        comp_surface = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use CondType11 method comp_surface: " + str(comp_surface)
                )
            )
        )
    else:
        comp_surface = comp_surface
    # cf Methods.Machine.CondType11.comp_width
    if isinstance(comp_width, ImportError):
        comp_width = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use CondType11 method comp_width: " + str(comp_width)
                )
            )
        )
    else:
        comp_width = comp_width
    # cf Methods.Machine.CondType11.plot
    if isinstance(plot, ImportError):
        plot = property(
            fget=lambda x: raise_(
                ImportError("Can't use CondType11 method plot: " + str(plot))
            )
        )
    else:
        plot = plot
    # cf Methods.Machine.CondType11.plot_schematics
    if isinstance(plot_schematics, ImportError):
        plot_schematics = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use CondType11 method plot_schematics: "
                    + str(plot_schematics)
                )
            )
        )
    else:
        plot_schematics = plot_schematics
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        Hwire=0.01,
        Wwire=0.01,
        Nwppc_rad=1,
        Nwppc_tan=1,
        Wins_wire=0,
        Wins_coil=0,
        type_winding_shape=0,
        alpha_ew=58,
        cond_mat=-1,
        ins_mat=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Hwire" in list(init_dict.keys()):
                Hwire = init_dict["Hwire"]
            if "Wwire" in list(init_dict.keys()):
                Wwire = init_dict["Wwire"]
            if "Nwppc_rad" in list(init_dict.keys()):
                Nwppc_rad = init_dict["Nwppc_rad"]
            if "Nwppc_tan" in list(init_dict.keys()):
                Nwppc_tan = init_dict["Nwppc_tan"]
            if "Wins_wire" in list(init_dict.keys()):
                Wins_wire = init_dict["Wins_wire"]
            if "Wins_coil" in list(init_dict.keys()):
                Wins_coil = init_dict["Wins_coil"]
            if "type_winding_shape" in list(init_dict.keys()):
                type_winding_shape = init_dict["type_winding_shape"]
            if "alpha_ew" in list(init_dict.keys()):
                alpha_ew = init_dict["alpha_ew"]
            if "cond_mat" in list(init_dict.keys()):
                cond_mat = init_dict["cond_mat"]
            if "ins_mat" in list(init_dict.keys()):
                ins_mat = init_dict["ins_mat"]
        # Set the properties (value check and convertion are done in setter)
        self.Hwire = Hwire
        self.Wwire = Wwire
        self.Nwppc_rad = Nwppc_rad
        self.Nwppc_tan = Nwppc_tan
        self.Wins_wire = Wins_wire
        self.Wins_coil = Wins_coil
        self.type_winding_shape = type_winding_shape
        self.alpha_ew = alpha_ew
        # Call Conductor init
        super(CondType11, self).__init__(cond_mat=cond_mat, ins_mat=ins_mat)
        # The class is frozen (in Conductor init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        CondType11_str = ""
        # Get the properties inherited from Conductor
        CondType11_str += super(CondType11, self).__str__()
        CondType11_str += "Hwire = " + str(self.Hwire) + linesep
        CondType11_str += "Wwire = " + str(self.Wwire) + linesep
        CondType11_str += "Nwppc_rad = " + str(self.Nwppc_rad) + linesep
        CondType11_str += "Nwppc_tan = " + str(self.Nwppc_tan) + linesep
        CondType11_str += "Wins_wire = " + str(self.Wins_wire) + linesep
        CondType11_str += "Wins_coil = " + str(self.Wins_coil) + linesep
        CondType11_str += (
            "type_winding_shape = " + str(self.type_winding_shape) + linesep
        )
        CondType11_str += "alpha_ew = " + str(self.alpha_ew) + linesep
        return CondType11_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Conductor
        if not super(CondType11, self).__eq__(other):
            return False
        if other.Hwire != self.Hwire:
            return False
        if other.Wwire != self.Wwire:
            return False
        if other.Nwppc_rad != self.Nwppc_rad:
            return False
        if other.Nwppc_tan != self.Nwppc_tan:
            return False
        if other.Wins_wire != self.Wins_wire:
            return False
        if other.Wins_coil != self.Wins_coil:
            return False
        if other.type_winding_shape != self.type_winding_shape:
            return False
        if other.alpha_ew != self.alpha_ew:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Conductor
        diff_list.extend(super(CondType11, self).compare(other, name=name))
        if other._Hwire != self._Hwire:
            diff_list.append(name + ".Hwire")
        if other._Wwire != self._Wwire:
            diff_list.append(name + ".Wwire")
        if other._Nwppc_rad != self._Nwppc_rad:
            diff_list.append(name + ".Nwppc_rad")
        if other._Nwppc_tan != self._Nwppc_tan:
            diff_list.append(name + ".Nwppc_tan")
        if other._Wins_wire != self._Wins_wire:
            diff_list.append(name + ".Wins_wire")
        if other._Wins_coil != self._Wins_coil:
            diff_list.append(name + ".Wins_coil")
        if other._type_winding_shape != self._type_winding_shape:
            diff_list.append(name + ".type_winding_shape")
        if other._alpha_ew != self._alpha_ew:
            diff_list.append(name + ".alpha_ew")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Conductor
        S += super(CondType11, self).__sizeof__()
        S += getsizeof(self.Hwire)
        S += getsizeof(self.Wwire)
        S += getsizeof(self.Nwppc_rad)
        S += getsizeof(self.Nwppc_tan)
        S += getsizeof(self.Wins_wire)
        S += getsizeof(self.Wins_coil)
        S += getsizeof(self.type_winding_shape)
        S += getsizeof(self.alpha_ew)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Conductor
        CondType11_dict = super(CondType11, self).as_dict(**kwargs)
        CondType11_dict["Hwire"] = self.Hwire
        CondType11_dict["Wwire"] = self.Wwire
        CondType11_dict["Nwppc_rad"] = self.Nwppc_rad
        CondType11_dict["Nwppc_tan"] = self.Nwppc_tan
        CondType11_dict["Wins_wire"] = self.Wins_wire
        CondType11_dict["Wins_coil"] = self.Wins_coil
        CondType11_dict["type_winding_shape"] = self.type_winding_shape
        CondType11_dict["alpha_ew"] = self.alpha_ew
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        CondType11_dict["__class__"] = "CondType11"
        return CondType11_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Hwire = None
        self.Wwire = None
        self.Nwppc_rad = None
        self.Nwppc_tan = None
        self.Wins_wire = None
        self.Wins_coil = None
        self.type_winding_shape = None
        self.alpha_ew = None
        # Set to None the properties inherited from Conductor
        super(CondType11, self)._set_None()

    def _get_Hwire(self):
        """getter of Hwire"""
        return self._Hwire

    def _set_Hwire(self, value):
        """setter of Hwire"""
        check_var("Hwire", value, "float", Vmin=0)
        self._Hwire = value

    Hwire = property(
        fget=_get_Hwire,
        fset=_set_Hwire,
        doc=u"""cf schematics, single wire height without insulation [m]

        :Type: float
        :min: 0
        """,
    )

    def _get_Wwire(self):
        """getter of Wwire"""
        return self._Wwire

    def _set_Wwire(self, value):
        """setter of Wwire"""
        check_var("Wwire", value, "float", Vmin=0)
        self._Wwire = value

    Wwire = property(
        fget=_get_Wwire,
        fset=_set_Wwire,
        doc=u"""cf schematics, single wire width without insulation [m]

        :Type: float
        :min: 0
        """,
    )

    def _get_Nwppc_rad(self):
        """getter of Nwppc_rad"""
        return self._Nwppc_rad

    def _set_Nwppc_rad(self, value):
        """setter of Nwppc_rad"""
        check_var("Nwppc_rad", value, "int", Vmin=1)
        self._Nwppc_rad = value

    Nwppc_rad = property(
        fget=_get_Nwppc_rad,
        fset=_set_Nwppc_rad,
        doc=u"""cf schematics, stator winding number of preformed wires (strands) in parallel per coil along radial (vertical) direction

        :Type: int
        :min: 1
        """,
    )

    def _get_Nwppc_tan(self):
        """getter of Nwppc_tan"""
        return self._Nwppc_tan

    def _set_Nwppc_tan(self, value):
        """setter of Nwppc_tan"""
        check_var("Nwppc_tan", value, "int", Vmin=1)
        self._Nwppc_tan = value

    Nwppc_tan = property(
        fget=_get_Nwppc_tan,
        fset=_set_Nwppc_tan,
        doc=u"""cf schematics, stator winding number of preformed wires (strands) in parallel per coil along tangential (horizontal) direction

        :Type: int
        :min: 1
        """,
    )

    def _get_Wins_wire(self):
        """getter of Wins_wire"""
        return self._Wins_wire

    def _set_Wins_wire(self, value):
        """setter of Wins_wire"""
        check_var("Wins_wire", value, "float", Vmin=0)
        self._Wins_wire = value

    Wins_wire = property(
        fget=_get_Wins_wire,
        fset=_set_Wins_wire,
        doc=u"""(advanced) cf schematics, winding strand insulation thickness [m]

        :Type: float
        :min: 0
        """,
    )

    def _get_Wins_coil(self):
        """getter of Wins_coil"""
        return self._Wins_coil

    def _set_Wins_coil(self, value):
        """setter of Wins_coil"""
        check_var("Wins_coil", value, "float", Vmin=0)
        self._Wins_coil = value

    Wins_coil = property(
        fget=_get_Wins_coil,
        fset=_set_Wins_coil,
        doc=u"""(advanced) cf schematics, winding coil insulation  thickness [m]

        :Type: float
        :min: 0
        """,
    )

    def _get_type_winding_shape(self):
        """getter of type_winding_shape"""
        return self._type_winding_shape

    def _set_type_winding_shape(self, value):
        """setter of type_winding_shape"""
        check_var("type_winding_shape", value, "int", Vmin=0, Vmax=1)
        self._type_winding_shape = value

    type_winding_shape = property(
        fget=_get_type_winding_shape,
        fset=_set_type_winding_shape,
        doc=u"""type of winding shape for end winding length calculation\n0 for hairpin windings\n1 for normal windings

        :Type: int
        :min: 0
        :max: 1
        """,
    )

    def _get_alpha_ew(self):
        """getter of alpha_ew"""
        return self._alpha_ew

    def _set_alpha_ew(self, value):
        """setter of alpha_ew"""
        check_var("alpha_ew", value, "float", Vmin=0, Vmax=180)
        self._alpha_ew = value

    alpha_ew = property(
        fget=_get_alpha_ew,
        fset=_set_alpha_ew,
        doc=u"""angle of winding overhang hairpin coils [deg]

        :Type: float
        :min: 0
        :max: 180
        """,
    )



# -*- coding: utf-8 -*-


def comp_surface_active(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotW27
        A SlotW27 object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """

    S2 = 0.5 * (self.W1 + self.W2) * self.H1
    S3 = 0.5 * (self.W2 + self.W3) * self.H2

    return S2 + S3



# -*- coding: utf-8 -*-

from ....Classes.OutElec import OutElec
from ....Classes.Simulation import Simulation
from ....Methods.Simulation.Input import InputError
from numpy import ndarray, pi, mean, transpose, zeros
from ....Functions.Electrical.coordinate_transformation import n2dq
from SciDataTool import Data1D, DataTime
from ....Functions.Winding.gen_phase_list import gen_name


def gen_input(self):
    """Generate the input for the magnetic module (electrical output)

    Parameters
    ----------
    self : InputCurrent
        An InputCurrent object
    """

    # Get the simulation
    if isinstance(self.parent, Simulation):
        simu = self.parent
    elif isinstance(self.parent.parent, Simulation):
        simu = self.parent.parent
    else:
        raise InputError(
            "ERROR: InputCurrent object should be inside a Simulation object"
        )

    # Create the correct Output object
    output = OutElec()

    # Set discretization
    Time, Angle = self.comp_axes(simu.machine, self.N0)
    output.Time = Time
    output.Angle = Angle

    # Number of winding phases for stator/rotor
    qs = len(simu.machine.stator.get_name_phase())
    qr = len(simu.machine.rotor.get_name_phase())

    output.N0 = self.N0
    output.felec = self.comp_felec()  # TODO introduce set_felec(slip)

    # Load and check Is
    if qs > 0:
        if self.Is is None:
            if self.Id_ref is None and self.Iq_ref is None:
                raise InputError(
                    "ERROR: InputCurrent.Is, InputCurrent.Id_ref, and InputCurrent.Iq_ref missing"
                )
            else:
                output.Id_ref = self.Id_ref
                output.Iq_ref = self.Iq_ref
                output.Is = None
        else:
            Is = self.Is.get_data()
            if not isinstance(Is, ndarray) or Is.shape != (self.Nt_tot, qs):
                raise InputError(
                    "ERROR: InputCurrent.Is must be a matrix with the shape "
                    + str((self.Nt_tot, qs))
                    + " (len(time), stator phase number), "
                    + str(Is.shape)
                    + " returned"
                )
            # Creating the data object
            Phase = Data1D(
                name="phase",
                unit="",
                values=gen_name(qs),
                is_components=True,
            )
            output.Is = DataTime(
                name="Stator current",
                unit="A",
                symbol="Is",
                axes=[Phase, Time],
                values=transpose(Is),
            )
            # Compute corresponding Id/Iq reference
            Idq = n2dq(
                transpose(output.Is.values),
                2 * pi * output.felec * output.Time.get_values(is_oneperiod=False),
                n=qs,
                is_dq_rms=True,
            )
            output.Id_ref = mean(Idq[:, 0])
            output.Iq_ref = mean(Idq[:, 1])  # TODO use of mean has to be documented

    # Load and check Ir is needed
    if qr > 0:
        if self.Ir is None:
            Ir = zeros((self.Nt_tot, qr))
        else:
            Ir = self.Ir.get_data()
        if not isinstance(Ir, ndarray) or Ir.shape != (self.Nt_tot, qr):
            raise InputError(
                "ERROR: InputCurrent.Ir must be a matrix with the shape "
                + str((self.Nt_tot, qr))
                + " (len(time), rotor phase number), "
                + str(Ir.shape)
                + " returned"
            )
        # Creating the data object
        Phase = Data1D(
            name="phase",
            unit="",
            values=gen_name(qr),
            is_components=True,
        )
        output.Ir = DataTime(
            name="Rotor current",
            unit="A",
            symbol="Ir",
            axes=[Phase, Time],
            values=transpose(Ir),
        )

    # Load and check alpha_rotor and N0
    if self.angle_rotor is None and self.N0 is None:
        raise InputError(
            "ERROR: InputCurrent.angle_rotor and InputCurrent.N0 can't be None at the same time"
        )
    if self.angle_rotor is not None:
        output.angle_rotor = self.angle_rotor.get_data()
        if (
            not isinstance(output.angle_rotor, ndarray)
            or len(output.angle_rotor.shape) != 1
            or output.angle_rotor.size != self.Nt_tot
        ):
            # angle_rotor should be a vector of same length as time
            raise InputError(
                "ERROR: InputCurrent.angle_rotor should be a vector of the same length as time, "
                + str(output.angle_rotor.shape)
                + " shape found, "
                + str(self.Nt_tot)
                + " expected"
            )

    if self.rot_dir is None or self.rot_dir not in [-1, 1]:
        # Enforce default rotation direction
        # simu.parent.geo.rot_dir = None
        pass  # None is already the default value
    else:
        simu.parent.geo.rot_dir = self.rot_dir

    if self.angle_rotor_initial is None:
        # Enforce default initial position
        output.angle_rotor_initial = 0
    else:
        output.angle_rotor_initial = self.angle_rotor_initial

    if self.Tem_av_ref is not None:
        output.Tem_av_ref = self.Tem_av_ref

    if simu.parent is None:
        raise InputError(
            "ERROR: The Simulation object must be in an Output object to run"
        )

    # Save the Output in the correct place
    simu.parent.elec = output



# -*- coding: utf-8 -*-

import numpy as np
from numpy import real, max as np_max

from ....Classes.MeshMat import MeshMat


def plot_glyph(
    self,
    label=None,
    index=None,
    indices=None,
    clim=None,
    factor=None,
    field_name=None,
    ifreq=0,
    save_path=None,
    is_point_arrow=False,
    group_names=None,
):
    """Plot the vector field as a glyph (or quiver) over the mesh.

    Parameters
    ----------
    self : MeshSolution
        a MeshSolution object
    label : str
        a label
    index : int
        an index
    indices : list
        list of the points to extract (optional)
    clim : list
        a list of 2 elements for the limits of the colorbar
    factor : float
        factor to multiply vector field
    field_name : str
        title of the field to display on plot
    ifreq : int
        index of the frequency to use for plot (if exists)
    save_path : str
        path to save the plot into an image
    is_point_arrow : bool
        to plot a nodal field (point-wise solution required)
    group_names : [str]
        plot is restricted to the group(s) corresponding to this list of group names.


    Returns
    -------
    """
    if group_names is not None:
        meshsol_grp = self.get_group(group_names)
        meshsol_grp.plot_glyph(
            label,
            index,
            indices,
            clim,
            factor,
            field_name,
            ifreq,
            save_path,
            is_point_arrow,
            None,
        )
    else:

        if save_path is None:
            try:
                import pyvistaqt as pv

                is_pyvistaqt = True
            except:
                import pyvista as pv

                is_pyvistaqt = False
        else:
            import pyvista as pv

            is_pyvistaqt = False

        # Get the mesh
        mesh = self.get_mesh(label=label, index=index)
        mesh_pv = mesh.get_mesh_pv(indices=indices)

        # Get the vector field
        args = dict()
        args["freq"] = [0]
        args["time"] = [0]
        vect_field = real(
            self.get_field(label=label, index=index, indices=indices, args=args)
        )
        # if len(vect_field.shape) == 3:
        #     # Third dimension is frequencies
        #     vect_field = vect_field[:, :, ifreq]

        # Compute factor
        if factor is None:
            factor = 1 / (100 * np_max(vect_field))

        if self.dimension == 2:
            vect_field = np.hstack((vect_field, np.zeros((vect_field.shape[0], 1))))

        # Add field to mesh
        if is_point_arrow:
            mesh_pv.vectors = vect_field * factor
            arrows_plt = mesh_pv.arrows
        else:
            mesh_pv["field"] = vect_field
            mesh_cell = mesh_pv.point_data_to_cell_data()
            surf = mesh_cell.extract_geometry()
            centers2 = surf.cell_centers()
            centers2.vectors = surf["field"] * factor
            arrows_plt = centers2.arrows

        # Configure plot
        if is_pyvistaqt:
            p = pv.BackgroundPlotter()
            p.set_background("white")
        else:
            pv.set_plot_theme("document")
            p = pv.Plotter(notebook=False)
        p.add_mesh(
            mesh_pv, color="grey", opacity=0.7, show_edges=True, edge_color="white"
        )
        p.add_mesh(arrows_plt, color="red")
        if self.dimension:
            p.view_xy()
        if save_path is None:
            p.show()
        else:
            p.show(interactive=False, screenshot=save_path)



# -*- coding: utf-8 -*-

from numpy import arcsin, pi

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.SlotW28 import *


def check(self):
    """Check that the SlotW28 object is correct

    Parameters
    ----------
    self : SlotW28
        A SlotW28 object

    Returns
    -------
    None

    Raises
    -------
    S28_RboW0CheckError
        You must have W0/2 < Rbo
    S28_R1W3CheckError
        W3 is too high comparing to the lamination bore radius (Rbo)
    S28_ZsCheckError
        There are too many slots to match W3 and R1
    """

    Rbo = self.get_Rbo()

    #    if self.R1 * 2 <= self.W0:
    #        raise S28_WCheckError("You must have W0 < 2*R1")

    if self.W0 * 0.5 / Rbo >= 1:
        raise S28_RboW0CheckError("You must have W0/2 < Rbo")

    if self.is_outwards():
        R1 = Rbo + self.H0 + self.R1
    else:
        R1 = Rbo - self.H0

    if self.W3 * 0.5 / R1 >= 1:
        raise S28_R1W3CheckError(
            "W3 is too high comparing to the lamination bore radius (Rbo)"
        )

    if self.R1 / R1 >= 1:
        raise S28_R1R1CheckError(
            "R1 is too high comparing to the lamination bore radius (Rbo)"
        )

    alpha = 2 * arcsin((2 * self.R1 + self.W3) * 0.5 / R1)

    if alpha > 2 * pi / self.Zs:
        raise S28_ZsCheckError("There are too many slots to match W3 and R1")



# -*- coding: utf-8 -*-
from ....Functions.Winding.gen_phase_list import gen_name


def get_name_phase(self):
    """Return the name of the winding phases

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object

    Returns
    -------
    name_phase: list
        List with the phase names

    """

    if self.winding is None:
        return list()
    return gen_name(self.winding.qs)



# -*- coding: utf-8 -*-

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.SlotW29 import *


def check(self):
    """Check that the SlotW29 object is correct

    Parameters
    ----------
    self : SlotW29
        A SlotW29 object

    Returns
    -------
    None

    Raises
    -------
    S29_W01CheckError
        You must have W0 < W1
    S29_W12CheckError
        You must have W1 < W2

    """
    if self.W1 <= self.W0:
        raise S29_W01CheckError("You must have W0 < W1")

    if self.W2 <= self.W1:
        raise S29_W12CheckError("You must have W1 < W2")



# -*- coding: utf-8 -*-


def comp_width(self):
    """Compute the width of the conductor

    Parameters
    ----------
    self : CondType11
        A CondType11 object

    Returns
    -------
    W: float
        Width of the conductor [m]

    """

    return (2 * self.Wins_wire + self.Wwire) * self.Nwppc_tan



# -*- coding: utf-8 -*-
from numpy import exp
from ....Methods.Geometry.Arc3 import AngleRotationArc3Error


def rotate(self, angle):
    """Rotation of the Arc3 of angle

    Parameters
    ----------
    self : Arc3
        An Arc3 Object

    angle : float
        the angle of rotation [rad]


    Returns
    -------
    None
    """
    if not isinstance(angle, float) and not isinstance(angle, int):
        raise AngleRotationArc3Error("The angle must be a float or int ")

    # check if Arc3 is correct"
    self.check()

    # Modification from the rotation  of the object
    self.begin = self.begin * exp(1j * angle)
    self.end = self.end * exp(1j * angle)



# -*- coding: utf-8 -*-

from ....Classes.MagFEMM import MagFEMM


def set_reused_data(self, simu, output, is_log=True, simu_type="VarSimu"):
    """Resuse some data from the reference simulation to skip computation

    Parameters
    ----------
    self : VarSimu
        a VarSimu object
    simu : Simulation
        The simulation to update
    output : Output
        Output from the reference simulation to enforce
    is_log : bool
        True to log the changes
    simu_type : str
        Name of the multi-simulation kind
    """

    if simu.layer == 2:
        TAB = "    "
    else:
        TAB = ""

    if self.is_reuse_femm_file and isinstance(simu.mag, MagFEMM):
        if is_log:
            self.get_logger().info(
                TAB
                + simu_type
                + ": Using same FEMM file for all simulation ("
                + output.mag.internal.FEMM_dict["path_save"]
                + ")"
            )
        simu.mag.import_file = output.mag.internal.FEMM_dict["path_save"]
        simu.mag.FEMM_dict_enforced = output.mag.internal.FEMM_dict



# -*- coding: utf-8 -*-

from deap.tools import selNSGA2
from copy import deepcopy
from datetime import datetime
import numpy as np

from ....Classes.Output import Output
from ....Classes.XOutput import XOutput
from ....Classes.DataKeeper import DataKeeper
from ....Classes.ParamExplorerSet import ParamExplorerSet
from ....Functions.Optimization.evaluate import evaluate
from ....Functions.Optimization.update import update
from ....Functions.Optimization.check_cstr import check_cstr
from ....Functions.Optimization.tournamentDCD import tournamentDCD


def create_setter(accessor, attribute):
    """
    Create a simulation setter
    """
    return lambda simu, val: setattr(eval(accessor), attribute, val)


def solve(self):
    """Method to perform NSGA-II using DEAP tools

    Parameters
    ----------
    self : OptiGenAlgNsga2Deap
        Solver to perform NSGA-II

    Returns
    -------
    multi_output : OutputMultiOpti
        class containing the results
    """

    logger = self.get_logger()

    # Check input parameters
    self.check_optimization_input()

    # Display information
    try:
        filename = self.get_logger().handlers[0].stream.name
        print(
            "{} Starting optimization... \n\tLog file: {}\n\tNumber of generations: {}\n\tPopulation size: {}\n".format(
                datetime.now().strftime("%H:%M:%S"),
                filename,
                self.nb_gen,
                self.size_pop,
            )
        )
    except (AttributeError, IndexError):
        print(
            "{} Starting optimization...\n\tNumber of generations: {}\n\tPopulation size: {}\n".format(
                datetime.now().strftime("%H:%M:%S"), self.nb_gen, self.size_pop
            )
        )

    try:
        # Keep number of evalutation to create the shape
        shape = self.size_pop

        # Create the toolbox
        self.create_toolbox()

        # Add the reference output to multi_output
        if isinstance(self.problem.simu.parent, Output):
            xoutput = XOutput(init_dict=self.problem.simu.parent.as_dict())
        else:
            xoutput = XOutput(simu=self.problem.simu.copy())

        self.xoutput = xoutput

        # Fitness symbol
        fitness_symbol = [of.symbol for of in self.problem.obj_func]

        # Set-up output data as list to be changed into ndarray at the end of the optimization
        paramexplorer_value = []
        xoutput.xoutput_dict["ngen"] = DataKeeper(
            name="Generation number", symbol="ngen"
        )
        xoutput.xoutput_dict["is_valid"] = DataKeeper(
            name="Individual validity", symbol="is_valid"
        )
        # Add datakeeper to XOutput to store additionnal values
        for dk in self.problem.datakeeper_list:
            assert dk.symbol not in xoutput.xoutput_dict
            xoutput.xoutput_dict[dk.symbol] = dk

        # Put objective functions in XOutput
        for obj_func in self.problem.obj_func:
            # obj_func is a DataKeeper instance
            xoutput.xoutput_dict[obj_func.symbol] = obj_func

        # Create the first population
        pop = self.toolbox.population(self.size_pop)

        # Evaluate the population
        nb_error = 0
        for i in range(0, self.size_pop):
            time = datetime.now().strftime("%H:%M:%S")
            print_gen_simu(time, 0, i, self.size_pop, nb_error, pop)
            nb_error += evaluate(self, pop[i])
            print_obj(self.problem.obj_func, pop[i])

        # Check the constraints violation
        nb_infeasible = 0
        if len(self.problem.constraint) > 0:
            time = datetime.now().strftime("%H:%M:%S")
            for indiv in pop:
                nb_infeasible += check_cstr(self, indiv) == False
        print(
            "\r{}  gen {:>5}: Finished, {:>4} errors,{:>4} infeasible.\n".format(
                time, 0, nb_error, nb_infeasible
            )
        )

        # Add pop to XOutput
        for indiv in pop:
            # Check that at every fitness values is different from inf
            is_valid = indiv.is_simu_valid and indiv.cstr_viol == 0

            if self.is_keep_all_output:
                xoutput.output_list.append(indiv.output)

            # is_valid
            xoutput.xoutput_dict["is_valid"].result.append(is_valid)

            # Design variable values
            paramexplorer_value.append(list(indiv))

            # Add fitness values to DataKeeper
            for i, symbol in enumerate(fitness_symbol):
                xoutput.xoutput_dict[symbol].result.append(indiv.fitness.values[i])

            # ngen
            xoutput.xoutput_dict["ngen"].result.append(0)

        if self.selector == None:
            pop = selNSGA2(pop, self.size_pop)
        else:
            parents = self.selector(pop, self.size_pop)

        ############################
        # LOOP FOR EACH GENERATION #
        ############################
        for ngen in range(1, self.nb_gen):
            # Extracting parents using
            parents = tournamentDCD(pop, self.size_pop)

            # Copy new indivuals
            children = []

            for indiv in parents:
                child = self.toolbox.individual()
                for i in range(len(indiv)):
                    child[i] = deepcopy(indiv[i])
                child.output = indiv.output.copy()
                child.fitness = deepcopy(indiv.fitness)
                children.append(child)

            for indiv1, indiv2 in zip(children[::2], children[::-2]):
                # Crossover
                is_cross = self.cross(indiv1, indiv2)

                # Mutation
                is_mutation = self.mutate(indiv1)
                if is_cross or is_mutation:
                    update(indiv1)

                is_mutation = self.mutate(indiv2)
                if is_cross or is_mutation:
                    update(indiv2)

            # Evaluate the children
            to_eval = []
            for indiv in children:
                if indiv.fitness.valid == False:
                    to_eval.append(indiv)

            shape += len(to_eval)

            nb_error = 0
            for i in range(len(to_eval)):
                time = datetime.now().strftime("%H:%M:%S")
                print_gen_simu(time, ngen, i, self.size_pop, nb_error, to_eval)
                nb_error += evaluate(self, to_eval[i])
                print_obj(self.problem.obj_func, to_eval[i])

            # Check the constraints violation
            nb_infeasible = 0
            if len(self.problem.constraint) > 0:
                time = datetime.now().strftime("%H:%M:%S")
                for indiv in to_eval:
                    nb_infeasible += check_cstr(self, indiv) == False
            print(
                "\r{}  gen {:>5}: Finished, {:>4} errors,{:>4} infeasible.\n".format(
                    time, ngen, nb_error, nb_infeasible
                )
            )

            # Add pop to XOutput
            for indiv in to_eval:
                # Check that at every fitness values is different from inf
                is_valid = indiv.is_simu_valid and indiv.cstr_viol == 0

                if self.is_keep_all_output:
                    xoutput.output_list.append(indiv.output)

                # is_valid
                xoutput.xoutput_dict["is_valid"].result.append(is_valid)

                # Design variable values
                paramexplorer_value.append(list(indiv))

                # Fitness values
                for i, symbol in enumerate(fitness_symbol):
                    xoutput.xoutput_dict[symbol].result.append(indiv.fitness.values[i])
                # ngen
                xoutput.xoutput_dict["ngen"].result.append(ngen)

            # Sorting the population according to NSGA2
            if self.selector == None:
                pop = selNSGA2(pop + children, self.size_pop)
            else:
                pop = self.selector(pop, self.size_pop)

        # Change xoutput variables in ndarray
        paramexplorer_value = np.array(paramexplorer_value)

        # Storing number of simulations
        xoutput.nb_simu = shape

        # Save design variable values in ParamExplorerSet
        for i, param_explorer in enumerate(self.problem.design_var):
            if param_explorer._setter_str is None:
                setter = param_explorer._setter_func
            else:
                setter = param_explorer._setter_str
            xoutput.paramexplorer_list.append(
                ParamExplorerSet(
                    name=param_explorer.name,
                    unit=param_explorer.unit,
                    symbol=param_explorer.symbol,
                    setter=setter,
                    value=paramexplorer_value[:, i].tolist(),
                )
            )

        # Delete toolbox so that classes created with DEAP remains after the optimization
        self.delete_toolbox()

        return xoutput

    except KeyboardInterrupt:
        # Except keybord interruption to return the results already computed
        logger.info("Interrupted by the user.")
        # Change xoutput variables in ndarray
        paramexplorer_value = np.array(paramexplorer_value)

        # Storing number of simulations
        xoutput.nb_simu = shape

        # Save design variable values in ParamExplorerSet
        for i, param_explorer in enumerate(self.problem.design_var):
            xoutput.paramexplorer_list.append(
                ParamExplorerSet(
                    name=param_explorer.name,
                    unit=param_explorer.unit,
                    symbol=param_explorer.symbol,
                    setter=param_explorer.setter,
                    value=paramexplorer_value[:, i].tolist(),
                )
            )

        # Delete toolbox so that classes created with DEAP remains after the optimization
        self.delete_toolbox()

        return xoutput

    except Exception as err:
        logger.error("{}: {}".format(type(err).__name__, err))
        raise err


def print_gen_simu(time, gen_id, simu_id, size_pop, nb_error, to_eval):
    print(
        "\r{}  gen {:>5}: simu {}/{} ({:>5.2f}%), {:>4} errors.".format(
            time,
            gen_id,
            (simu_id + 1),
            size_pop,
            (simu_id) * 100 / size_pop,
            nb_error,
        )
    )
    msg = "Design Variables: "
    for ii in range(len(to_eval[simu_id])):
        msg += (
            to_eval[simu_id].design_var[ii].symbol
            + ": "
            + format(to_eval[simu_id][ii], ".2e")
            + ", "
        )
    print(msg[:-2])


def print_obj(obj_func, indiv):
    msg = "Objectives: "
    for ii in range(len(obj_func)):
        msg += (
            obj_func[ii].symbol + ": " + format(indiv.fitness.values[ii], ".2e") + ", "
        )
    print(msg[:-2] + "\n")



# -*- coding: utf-8 -*-


def get_pole_pair_number(self):
    """Returns the number of pole pairs of the machine

    Parameters
    ----------
    self : Machine
        Machine object

    Returns
    -------
    p: int
        Pole pair number of the machine
    """

    # Get list of laminations
    lam_list = self.get_lam_list()

    p = lam_list.pop().get_pole_pair_number()

    # Get pole pair number for all laminations
    for i, lam in enumerate(lam_list):
        # Check that the pole pair number of lam is the same as the previous one
        if lam.get_pole_pair_number() != p:
            raise Exception(
                "ERROR, lamination "
                + str(i + 1)
                + " has a different pole pair number than lamination 0"
            )

    return p



# -*- coding: utf-8 -*-

# File generated according to WVent.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_WVent(object):
    def setupUi(self, WVent):
        if not WVent.objectName():
            WVent.setObjectName(u"WVent")
        WVent.resize(630, 470)
        WVent.setMinimumSize(QSize(630, 470))
        self.main_layout = QVBoxLayout(WVent)
        self.main_layout.setObjectName(u"main_layout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.in_vent_type = QLabel(WVent)
        self.in_vent_type.setObjectName(u"in_vent_type")

        self.horizontalLayout.addWidget(self.in_vent_type)

        self.c_vent_type = QComboBox(WVent)
        self.c_vent_type.addItem("")
        self.c_vent_type.addItem("")
        self.c_vent_type.addItem("")
        self.c_vent_type.setObjectName(u"c_vent_type")

        self.horizontalLayout.addWidget(self.c_vent_type)

        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.main_layout.addLayout(self.horizontalLayout)

        self.w_vent = QWidget(WVent)
        self.w_vent.setObjectName(u"w_vent")
        self.w_vent.setMinimumSize(QSize(640, 480))

        self.main_layout.addWidget(self.w_vent)

        self.retranslateUi(WVent)

        QMetaObject.connectSlotsByName(WVent)

    # setupUi

    def retranslateUi(self, WVent):
        WVent.setWindowTitle(QCoreApplication.translate("WVent", u"Form", None))
        self.in_vent_type.setText(
            QCoreApplication.translate("WVent", u"Ventilation Shape:", None)
        )
        self.c_vent_type.setItemText(
            0, QCoreApplication.translate("WVent", u"Circular", None)
        )
        self.c_vent_type.setItemText(
            1, QCoreApplication.translate("WVent", u"Polar", None)
        )
        self.c_vent_type.setItemText(
            2, QCoreApplication.translate("WVent", u"Trapeze", None)
        )

    # retranslateUi



from ....Methods.Simulation.Input import InputError


def comp_felec(self):
    """Compute the electrical frequency

    Parameters
    ----------
    felec : float
        Electrical frequency [Hz]
    """

    if hasattr(self, "felec") and self.felec is not None:
        return self.felec  # TODO maybe add some checks?
    elif self.N0 is not None:
        # Get the phase number for verifications
        if self.parent is None:
            raise InputError(
                "ERROR: InputCurrent object should be inside a Simulation object"
            )
        # get the pole pair number
        if hasattr(self.parent, "machine"):
            zp = self.parent.machine.stator.get_pole_pair_number()
        elif hasattr(self.parent.parent, "machine"):
            zp = self.parent.parent.machine.stator.get_pole_pair_number()
        else:
            logger = self.get_logger()
            logger.warning("Input.comp_felec(): Machine was not found.")
            zp = 1
        return self.N0 * zp / 60
    else:
        raise InputError("ERROR: InputCurrent object can't have felec and N0 at None")



# -*- coding: utf-8 -*-


def get_length(self):
    """Return the length of the Frame

    Parameters
    ----------
    self : Frame
        A Frame object

    Returns
    -------
    length: float
        Length of the frame

    """

    if self.comp_height_eq() > 0:
        return self.Lfra
    else:
        return 0



# -*- coding: utf-8 -*-
from ....Classes.PointMat import PointMat
from ....Classes.CellMat import CellMat
from ....Classes.Interpolation import Interpolation
from ....Classes.FPGNSeg import FPGNSeg
from ....Classes.ScalarProductL2 import ScalarProductL2
from ....Classes.RefSegmentP1 import RefSegmentP1

from ....definitions import PACKAGE_NAME
from collections import Counter
import numpy as np
from itertools import combinations


def interface(self, other_mesh):
    """Define a MeshMat object corresponding to the exact intersection between two meshes (points must be in both meshes).

    Parameters
    ----------
    self : MeshMat
        a Mesh object
    other_mesh : Mesh
        an other Mesh object

        Returns
    -------
    """

    # Dynamic import
    new_mesh = self.copy()
    # new_mesh.point = PointMat()
    new_mesh.cell = dict()

    for key in self.cell:

        # Developer info: IDK if this code works with other than triangle cells. To be checked.
        if self.cell[key].nb_pt_per_cell == 3:  # Triangle case

            new_mesh.cell["line"] = CellMat(nb_pt_per_cell=2)
            interp = Interpolation()
            interp.gauss_point = FPGNSeg()
            interp.ref_cell = RefSegmentP1()
            interp.scalar_product = ScalarProductL2()
            new_mesh.cell["line"].interpolation = interp

            connect = self.cell[key].get_connectivity()
            connect2 = other_mesh.cell[key].get_connectivity()

            points_tags = np.unique(connect)
            other_points_tags = np.unique(connect2)

            # Find the points on the interface (they are in both in and out)
            interface_points_tags = np.intersect1d(points_tags, other_points_tags)
            nb_interf_points = len(interface_points_tags)

            comb = combinations(range(self.cell[key].nb_pt_per_cell), 2)

            for duo in list(comb):
                col1i = np.mod(duo[0], 3)
                col2i = np.mod(duo[1], 3)
                col1 = connect[:, col1i]
                col2 = connect[:, col2i]

                col1_bin = np.zeros(len(col1))
                col2_bin = np.zeros(len(col1))

                for pt in interface_points_tags:
                    Icol1i = np.where(col1 == pt)[0]
                    Icol2i = np.where(col2 == pt)[0]
                    col1_bin[Icol1i] = 1
                    col2_bin[Icol2i] = 1

                # Position in vector where 2 nodes of the same element are on the interface (potential line element)
                I_target = np.where(col1_bin + col2_bin == 2)[0]

                comb2 = combinations(range(other_mesh.cell[key].nb_pt_per_cell), 2)
                for duo2 in list(comb2):
                    col1j = np.mod(duo2[0], 3)
                    col2j = np.mod(duo2[1], 3)
                    col12 = connect2[:, col1j]
                    col22 = connect2[:, col2j]

                    col12_bin = np.zeros(len(col12))
                    col22_bin = np.zeros(len(col12))

                    for pt in interface_points_tags:
                        Icol1i = np.where(col12 == pt)[0]
                        Icol2i = np.where(col22 == pt)[0]
                        col12_bin[Icol1i] = 1
                        col22_bin[Icol2i] = 1

                    # Same but in the second mesh
                    I_target2 = np.where(col12_bin + col22_bin == 2)[0]

                    for itag in I_target2:
                        e_tag1 = col12[itag]
                        e_tag2 = col22[itag]
                        Iline = np.where(
                            ((col1[I_target] == e_tag1) & (col2[I_target] == e_tag2))
                            | ((col2[I_target] == e_tag1) & (col1[I_target] == e_tag2))
                        )[0]
                        if Iline.size != 0:
                            new_mesh.add_cell([e_tag1, e_tag2], "line")

    return new_mesh
    # TODO : Extend the code to higher dimension (3 points triangles for tetrahedra interfaces ...)

    # import matplotlib.pyplot as plt
    # fig, ax = plt.subplots()
    # x = mesh_group.point[:,0]
    # y = mesh_group.point[:,1]
    # ax.scatter(x, y)
    # for ieleme in range(mesh_group.nb_elem):
    #     x = mesh_group.point[interface_elem[ieleme,:],0]
    #     y = mesh_group.point[interface_elem[ieleme,:],1]
    #     ax.plot(x, y)
    #
    # fig, ax = plt.subplots()
    # x = points_in[:, 0]
    # y = points_in[:, 1]
    # ax.scatter(x, y)
    # x = points_out[:, 0]
    # y = points_out[:, 1]
    # ax.scatter(x, y)
    # x = points_parent[interface_points_id, 0]
    # y = points_parent[interface_points_id, 1]
    # ax.scatter(x, y, marker='x')



from numpy import ndarray, min as np_min, max as np_max

from SciDataTool import VectorField, Data
from ....Functions.Load.import_class import import_class
from ....Functions.Simulation.VarSimu.log_datakeeper_step_result import (
    log_datakeeper_step_result,
)
from logging import WARNING
from ....loggers import CONSOLE_LEVEL


def run_multisim_step(
    simulation,
    datakeeper_list,
    stop_if_error,
    post_keeper_postproc_list=None,
    simu_type=None,
):
    """
    Execute a simulation step of a multi-simulation and run datakeepers

    Parameters:
    -----------
    simulation: Simulation
        Simulation to run
    datakeeper_list
        List of datakeeper to run and update results
    stop_if_error: bool
        True: Raises an error if the simulation fails
    post_keeper_postproc_list : list
        list of postprocessing to run after the datakeeper
    simu_type : str
        To adapt the text ex: "Variable Load Results"
    """
    # Change console logger level from a specific layer (if requested)
    if (
        simulation.layer_log_warn is not None
        and simulation.layer >= simulation.layer_log_warn
    ):
        simulation.get_logger().handlers[0].level = WARNING

    # Run the simulation with/without error handling
    if stop_if_error:
        is_error = False
        result = simulation.run()
    else:
        try:
            result = simulation.run()
            is_error = False
        except Exception as err:
            print(err)
            is_error = True
            result = None

    # Revert logger level
    simulation.get_logger().handlers[0].level = CONSOLE_LEVEL

    # Datakeepers
    index = simulation.index
    if is_error:  # Execute error_keeper
        for datakeeper in datakeeper_list:
            if datakeeper.error_keeper is None:
                value = None
            else:
                value = datakeeper.error_keeper(simulation)
            if simulation.index is None:  # index == None => ref simu
                datakeeper.result_ref = value
            else:
                datakeeper.result[index] = value
    else:  # Execute Normal DataKeeper
        for datakeeper in datakeeper_list:
            # Run and store Datakeeper
            try:
                value = datakeeper.keeper(result)
            except Exception as e:
                simulation.get_logger().error(
                    "ERROR while calling DataKeeper "
                    + datakeeper.name
                    + " for simulation "
                    + str(index)
                    + ":\n"
                    + str(e)
                )
                if stop_if_error:
                    raise e
                value = datakeeper.error_keeper(simulation)
            if simulation.index is None:  # index == None => ref simu
                datakeeper.result_ref = value
            else:
                datakeeper.result[index] = value
        log_datakeeper_step_result(simulation, datakeeper_list, index, simu_type)

        # Run Post datakeeper post-processings
        if post_keeper_postproc_list is not None:
            for postproc in post_keeper_postproc_list:
                postproc.run(result)
    return result



def get_data_from_str(self, data_str):
    """Return the data object corresponding to the data str

    Parameters
    ----------
    self : Output
        an Output object
    data_str : string
        a string corresponding to a data object

    Returns
    -------
    data : Data
        a Data object

    """

    # Get Data object names
    phys = getattr(self, data_str.split(".")[0])
    if "get_" in data_str.split(".")[1]:  # get method
        data = getattr(phys, data_str.split(".")[1])()
    elif "[" in data_str.split(".")[1]:  # dict
        d = getattr(phys, data_str.split(".")[1].split("[")[0])
        data = d[data_str.split(".")[1].split("[")[1].rstrip("]")]
    else:
        data = getattr(phys, data_str.split(".")[1])

    return data



# -*- coding: utf-8 -*-

# File generated according to SBar.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from .....GUI.Tools.FloatEdit import FloatEdit
from .....GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SBar(object):
    def setupUi(self, SBar):
        if not SBar.objectName():
            SBar.setObjectName(u"SBar")
        SBar.resize(650, 550)
        SBar.setMinimumSize(QSize(650, 550))
        self.verticalLayout_2 = QVBoxLayout(SBar)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.g_bar = QGroupBox(SBar)
        self.g_bar.setObjectName(u"g_bar")
        self.verticalLayout = QVBoxLayout(self.g_bar)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.c_bar_type = QComboBox(self.g_bar)
        self.c_bar_type.addItem("")
        self.c_bar_type.addItem("")
        self.c_bar_type.setObjectName(u"c_bar_type")
        self.c_bar_type.setMinimumSize(QSize(100, 0))

        self.horizontalLayout.addWidget(self.c_bar_type)

        self.horizontalSpacer_2 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer_2)

        self.verticalLayout.addLayout(self.horizontalLayout)

        self.w_bar = QWidget(self.g_bar)
        self.w_bar.setObjectName(u"w_bar")

        self.verticalLayout.addWidget(self.w_bar)

        self.verticalLayout_2.addWidget(self.g_bar)

        self.g_ring = QGroupBox(SBar)
        self.g_ring.setObjectName(u"g_ring")
        self.horizontalLayout_2 = QHBoxLayout(self.g_ring)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.img_ring = QLabel(self.g_ring)
        self.img_ring.setObjectName(u"img_ring")
        self.img_ring.setMaximumSize(QSize(16777215, 16777215))
        self.img_ring.setPixmap(QPixmap(u":/images/images/MachineSetup/Bar/Ring.PNG"))
        self.img_ring.setScaledContents(True)

        self.horizontalLayout_2.addWidget(self.img_ring)

        self.widget = QWidget(self.g_ring)
        self.widget.setObjectName(u"widget")
        self.widget.setMinimumSize(QSize(250, 0))
        self.widget.setMaximumSize(QSize(250, 16777215))
        self.verticalLayout_3 = QVBoxLayout(self.widget)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.lf_Lewout = FloatEdit(self.widget)
        self.lf_Lewout.setObjectName(u"lf_Lewout")
        self.lf_Lewout.setMinimumSize(QSize(70, 0))
        self.lf_Lewout.setMaximumSize(QSize(100, 20))

        self.gridLayout.addWidget(self.lf_Lewout, 2, 1, 1, 1)

        self.unit_Hscr = QLabel(self.widget)
        self.unit_Hscr.setObjectName(u"unit_Hscr")
        self.unit_Hscr.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.unit_Hscr, 0, 2, 1, 1)

        self.unit_Lscr = QLabel(self.widget)
        self.unit_Lscr.setObjectName(u"unit_Lscr")
        self.unit_Lscr.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.unit_Lscr, 1, 2, 1, 1)

        self.in_Hscr = QLabel(self.widget)
        self.in_Hscr.setObjectName(u"in_Hscr")
        self.in_Hscr.setMinimumSize(QSize(40, 0))

        self.gridLayout.addWidget(self.in_Hscr, 0, 0, 1, 1)

        self.lf_Hscr = FloatEdit(self.widget)
        self.lf_Hscr.setObjectName(u"lf_Hscr")
        self.lf_Hscr.setMinimumSize(QSize(70, 0))
        self.lf_Hscr.setMaximumSize(QSize(100, 20))

        self.gridLayout.addWidget(self.lf_Hscr, 0, 1, 1, 1)

        self.lf_Lscr = FloatEdit(self.widget)
        self.lf_Lscr.setObjectName(u"lf_Lscr")
        self.lf_Lscr.setMinimumSize(QSize(70, 0))
        self.lf_Lscr.setMaximumSize(QSize(100, 20))

        self.gridLayout.addWidget(self.lf_Lscr, 1, 1, 1, 1)

        self.unit_Lewout = QLabel(self.widget)
        self.unit_Lewout.setObjectName(u"unit_Lewout")
        self.unit_Lewout.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.unit_Lewout, 2, 2, 1, 1)

        self.in_Lewout = QLabel(self.widget)
        self.in_Lewout.setObjectName(u"in_Lewout")
        self.in_Lewout.setMinimumSize(QSize(40, 0))

        self.gridLayout.addWidget(self.in_Lewout, 2, 0, 1, 1)

        self.in_Lscr = QLabel(self.widget)
        self.in_Lscr.setObjectName(u"in_Lscr")
        self.in_Lscr.setMinimumSize(QSize(40, 0))

        self.gridLayout.addWidget(self.in_Lscr, 1, 0, 1, 1)

        self.verticalLayout_3.addLayout(self.gridLayout)

        self.w_mat = WMatSelect(self.widget)
        self.w_mat.setObjectName(u"w_mat")

        self.verticalLayout_3.addWidget(self.w_mat)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_3.addItem(self.verticalSpacer)

        self.horizontalLayout_2.addWidget(self.widget)

        self.verticalLayout_2.addWidget(self.g_ring)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalSpacer_3 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_3.addItem(self.horizontalSpacer_3)

        self.b_plot = QPushButton(SBar)
        self.b_plot.setObjectName(u"b_plot")

        self.horizontalLayout_3.addWidget(self.b_plot)

        self.b_previous = QPushButton(SBar)
        self.b_previous.setObjectName(u"b_previous")

        self.horizontalLayout_3.addWidget(self.b_previous)

        self.b_next = QPushButton(SBar)
        self.b_next.setObjectName(u"b_next")
        self.b_next.setEnabled(True)

        self.horizontalLayout_3.addWidget(self.b_next)

        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        QWidget.setTabOrder(self.lf_Hscr, self.lf_Lscr)
        QWidget.setTabOrder(self.lf_Lscr, self.lf_Lewout)
        QWidget.setTabOrder(self.lf_Lewout, self.b_previous)
        QWidget.setTabOrder(self.b_previous, self.b_plot)
        QWidget.setTabOrder(self.b_plot, self.c_bar_type)

        self.retranslateUi(SBar)

        QMetaObject.connectSlotsByName(SBar)

    # setupUi

    def retranslateUi(self, SBar):
        SBar.setWindowTitle(QCoreApplication.translate("SBar", u"Form", None))
        self.g_bar.setTitle(QCoreApplication.translate("SBar", u"Bar", None))
        self.c_bar_type.setItemText(
            0, QCoreApplication.translate("SBar", u"Rectangular bar", None)
        )
        self.c_bar_type.setItemText(
            1, QCoreApplication.translate("SBar", u"Die cast bar", None)
        )

        self.g_ring.setTitle(
            QCoreApplication.translate("SBar", u"Short Circuit Ring", None)
        )
        self.img_ring.setText("")
        self.unit_Hscr.setText(QCoreApplication.translate("SBar", u"m", None))
        self.unit_Lscr.setText(QCoreApplication.translate("SBar", u"m", None))
        self.in_Hscr.setText(QCoreApplication.translate("SBar", u"Hscr :", None))
        self.unit_Lewout.setText(QCoreApplication.translate("SBar", u"m", None))
        self.in_Lewout.setText(QCoreApplication.translate("SBar", u"Lewout :", None))
        self.in_Lscr.setText(QCoreApplication.translate("SBar", u"Lscr :", None))
        self.b_plot.setText(QCoreApplication.translate("SBar", u"Preview", None))
        self.b_previous.setText(QCoreApplication.translate("SBar", u"Previous", None))
        self.b_next.setText(QCoreApplication.translate("SBar", u"Save", None))

    # retranslateUi



from ....Functions.Electrical.coordinate_transformation import dq2n
from numpy import pi, array, transpose
from SciDataTool import Data1D, DataTime
from ....Functions.Winding.gen_phase_list import gen_name


def get_Us(self):
    """Return the stator voltage"""
    if self.Us is None:
        # Generate current according to Ud/Uq
        Usdq = array([self.Ud_ref, self.Uq_ref])
        time = self.Time.get_values(is_oneperiod=True)
        qs = self.parent.simu.machine.stator.winding.qs
        felec = self.felec

        # add stator current
        Us = dq2n(Usdq, 2 * pi * felec * time, n=qs)
        Phase = Data1D(
            name="phase",
            unit="",
            values=gen_name(qs),
            is_components=True,
        )
        self.Us = DataTime(
            name="Stator voltage",
            unit="V",
            symbol="Us",
            axes=[Phase, self.Time.copy()],
            values=transpose(Us),
        )
    return self.Us



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Mesh/MeshMat.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Mesh/MeshMat
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Mesh import Mesh

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Mesh.MeshMat.get_point import get_point
except ImportError as error:
    get_point = error

try:
    from ..Methods.Mesh.MeshMat.get_cell import get_cell
except ImportError as error:
    get_cell = error

try:
    from ..Methods.Mesh.MeshMat.get_mesh_pv import get_mesh_pv
except ImportError as error:
    get_mesh_pv = error

try:
    from ..Methods.Mesh.MeshMat.get_cell_area import get_cell_area
except ImportError as error:
    get_cell_area = error

try:
    from ..Methods.Mesh.MeshMat.add_cell import add_cell
except ImportError as error:
    add_cell = error

try:
    from ..Methods.Mesh.MeshMat.get_vertice import get_vertice
except ImportError as error:
    get_vertice = error

try:
    from ..Methods.Mesh.MeshMat.get_point2cell import get_point2cell
except ImportError as error:
    get_point2cell = error

try:
    from ..Methods.Mesh.MeshMat.renum import renum
except ImportError as error:
    renum = error

try:
    from ..Methods.Mesh.MeshMat.find_cell import find_cell
except ImportError as error:
    find_cell = error

try:
    from ..Methods.Mesh.MeshMat.interface import interface
except ImportError as error:
    interface = error


from ._check import InitUnKnowClassError
from .CellMat import CellMat
from .PointMat import PointMat


class MeshMat(Mesh):
    """Gather the mesh storage format"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Mesh.MeshMat.get_point
    if isinstance(get_point, ImportError):
        get_point = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshMat method get_point: " + str(get_point))
            )
        )
    else:
        get_point = get_point
    # cf Methods.Mesh.MeshMat.get_cell
    if isinstance(get_cell, ImportError):
        get_cell = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshMat method get_cell: " + str(get_cell))
            )
        )
    else:
        get_cell = get_cell
    # cf Methods.Mesh.MeshMat.get_mesh_pv
    if isinstance(get_mesh_pv, ImportError):
        get_mesh_pv = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshMat method get_mesh_pv: " + str(get_mesh_pv))
            )
        )
    else:
        get_mesh_pv = get_mesh_pv
    # cf Methods.Mesh.MeshMat.get_cell_area
    if isinstance(get_cell_area, ImportError):
        get_cell_area = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MeshMat method get_cell_area: " + str(get_cell_area)
                )
            )
        )
    else:
        get_cell_area = get_cell_area
    # cf Methods.Mesh.MeshMat.add_cell
    if isinstance(add_cell, ImportError):
        add_cell = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshMat method add_cell: " + str(add_cell))
            )
        )
    else:
        add_cell = add_cell
    # cf Methods.Mesh.MeshMat.get_vertice
    if isinstance(get_vertice, ImportError):
        get_vertice = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshMat method get_vertice: " + str(get_vertice))
            )
        )
    else:
        get_vertice = get_vertice
    # cf Methods.Mesh.MeshMat.get_point2cell
    if isinstance(get_point2cell, ImportError):
        get_point2cell = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MeshMat method get_point2cell: " + str(get_point2cell)
                )
            )
        )
    else:
        get_point2cell = get_point2cell
    # cf Methods.Mesh.MeshMat.renum
    if isinstance(renum, ImportError):
        renum = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshMat method renum: " + str(renum))
            )
        )
    else:
        renum = renum
    # cf Methods.Mesh.MeshMat.find_cell
    if isinstance(find_cell, ImportError):
        find_cell = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshMat method find_cell: " + str(find_cell))
            )
        )
    else:
        find_cell = find_cell
    # cf Methods.Mesh.MeshMat.interface
    if isinstance(interface, ImportError):
        interface = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshMat method interface: " + str(interface))
            )
        )
    else:
        interface = interface
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self, cell=-1, point=-1, label=None, dimension=2, init_dict=None, init_str=None
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "cell" in list(init_dict.keys()):
                cell = init_dict["cell"]
            if "point" in list(init_dict.keys()):
                point = init_dict["point"]
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
            if "dimension" in list(init_dict.keys()):
                dimension = init_dict["dimension"]
        # Set the properties (value check and convertion are done in setter)
        self.cell = cell
        self.point = point
        # Call Mesh init
        super(MeshMat, self).__init__(label=label, dimension=dimension)
        # The class is frozen (in Mesh init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        MeshMat_str = ""
        # Get the properties inherited from Mesh
        MeshMat_str += super(MeshMat, self).__str__()
        if len(self.cell) == 0:
            MeshMat_str += "cell = dict()" + linesep
        for key, obj in self.cell.items():
            tmp = self.cell[key].__str__().replace(linesep, linesep + "\t") + linesep
            MeshMat_str += "cell[" + key + "] =" + tmp + linesep + linesep
        if self.point is not None:
            tmp = self.point.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            MeshMat_str += "point = " + tmp
        else:
            MeshMat_str += "point = None" + linesep + linesep
        return MeshMat_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Mesh
        if not super(MeshMat, self).__eq__(other):
            return False
        if other.cell != self.cell:
            return False
        if other.point != self.point:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Mesh
        diff_list.extend(super(MeshMat, self).compare(other, name=name))
        if (other.cell is None and self.cell is not None) or (
            other.cell is not None and self.cell is None
        ):
            diff_list.append(name + ".cell None mismatch")
        elif self.cell is None:
            pass
        elif len(other.cell) != len(self.cell):
            diff_list.append("len(" + name + "cell)")
        else:
            for key in self.cell:
                diff_list.extend(
                    self.cell[key].compare(other.cell[key], name=name + ".cell")
                )
        if (other.point is None and self.point is not None) or (
            other.point is not None and self.point is None
        ):
            diff_list.append(name + ".point None mismatch")
        elif self.point is not None:
            diff_list.extend(self.point.compare(other.point, name=name + ".point"))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Mesh
        S += super(MeshMat, self).__sizeof__()
        if self.cell is not None:
            for key, value in self.cell.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.point)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Mesh
        MeshMat_dict = super(MeshMat, self).as_dict(**kwargs)
        if self.cell is None:
            MeshMat_dict["cell"] = None
        else:
            MeshMat_dict["cell"] = dict()
            for key, obj in self.cell.items():
                if obj is not None:
                    MeshMat_dict["cell"][key] = obj.as_dict(**kwargs)
                else:
                    MeshMat_dict["cell"][key] = None
        if self.point is None:
            MeshMat_dict["point"] = None
        else:
            MeshMat_dict["point"] = self.point.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        MeshMat_dict["__class__"] = "MeshMat"
        return MeshMat_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.cell = None
        if self.point is not None:
            self.point._set_None()
        # Set to None the properties inherited from Mesh
        super(MeshMat, self)._set_None()

    def _get_cell(self):
        """getter of cell"""
        if self._cell is not None:
            for key, obj in self._cell.items():
                if obj is not None:
                    obj.parent = self
        return self._cell

    def _set_cell(self, value):
        """setter of cell"""
        if type(value) is dict:
            for key, obj in value.items():
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "cell"
                    )
                    value[key] = class_obj(init_dict=obj)
        if type(value) is int and value == -1:
            value = dict()
        check_var("cell", value, "{CellMat}")
        self._cell = value

    cell = property(
        fget=_get_cell,
        fset=_set_cell,
        doc=u"""Storing connectivity

        :Type: {CellMat}
        """,
    )

    def _get_point(self):
        """getter of point"""
        return self._point

    def _set_point(self, value):
        """setter of point"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "point"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = PointMat()
        check_var("point", value, "PointMat")
        self._point = value

        if self._point is not None:
            self._point.parent = self

    point = property(
        fget=_get_point,
        fset=_set_point,
        doc=u"""Storing nodes

        :Type: PointMat
        """,
    )



# -*- coding: utf-8 -*-
def get_lines(self):
    """The list returned contains all the Line of the SurRing

    Parameters
    ----------
    self : SurfRing
        A SurfRing object


    Returns
    -------
    line_list : list
        list of lines delimiting the surface

    """
    line_list = self.out_surf.get_lines()
    line_list.extend(self.in_surf.get_lines())
    return line_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/ParamExplorerSet.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/ParamExplorerSet
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .ParamExplorer import ParamExplorer

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.ParamExplorerSet.get_value import get_value
except ImportError as error:
    get_value = error

try:
    from ..Methods.Simulation.ParamExplorerSet.as_dict import as_dict
except ImportError as error:
    as_dict = error


from ntpath import basename
from os.path import isfile
from ._check import CheckTypeError
import numpy as np
import random
from ._check import InitUnKnowClassError


class ParamExplorerSet(ParamExplorer):
    """Abstract class for the multi-simulation"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.ParamExplorerSet.get_value
    if isinstance(get_value, ImportError):
        get_value = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ParamExplorerSet method get_value: " + str(get_value)
                )
            )
        )
    else:
        get_value = get_value
    # cf Methods.Simulation.ParamExplorerSet.as_dict
    if isinstance(as_dict, ImportError):
        as_dict = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ParamExplorerSet method as_dict: " + str(as_dict)
                )
            )
        )
    else:
        as_dict = as_dict
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        value=-1,
        name="",
        symbol="",
        unit="",
        setter=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "value" in list(init_dict.keys()):
                value = init_dict["value"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
            if "setter" in list(init_dict.keys()):
                setter = init_dict["setter"]
        # Set the properties (value check and convertion are done in setter)
        self.value = value
        # Call ParamExplorer init
        super(ParamExplorerSet, self).__init__(
            name=name, symbol=symbol, unit=unit, setter=setter
        )
        # The class is frozen (in ParamExplorer init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        ParamExplorerSet_str = ""
        # Get the properties inherited from ParamExplorer
        ParamExplorerSet_str += super(ParamExplorerSet, self).__str__()
        ParamExplorerSet_str += (
            "value = "
            + linesep
            + str(self.value).replace(linesep, linesep + "\t")
            + linesep
        )
        return ParamExplorerSet_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from ParamExplorer
        if not super(ParamExplorerSet, self).__eq__(other):
            return False
        if other.value != self.value:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from ParamExplorer
        diff_list.extend(super(ParamExplorerSet, self).compare(other, name=name))
        if other._value != self._value:
            diff_list.append(name + ".value")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from ParamExplorer
        S += super(ParamExplorerSet, self).__sizeof__()
        if self.value is not None:
            for value in self.value:
                S += getsizeof(value)
        return S

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.value = None
        # Set to None the properties inherited from ParamExplorer
        super(ParamExplorerSet, self)._set_None()

    def _get_value(self):
        """getter of value"""
        return self._value

    def _set_value(self, value):
        """setter of value"""
        if type(value) is int and value == -1:
            value = list()
        check_var("value", value, "list")
        self._value = value

    value = property(
        fget=_get_value,
        fset=_set_value,
        doc=u"""List containing the different parameter values to explore

        :Type: list
        """,
    )



from numpy import pi
from ...Classes.Arc import Arc
import sys
import gmsh


def gen_3D_mesh(
    lamination,
    save_path="Lamination.msh",
    sym=-1,
    mesh_size=5e-3,
    user_mesh_dict=None,
    is_rect=False,
    Nlayer=20,
    display=True,
):
    """Draw 3D mesh of the lamination
    Parameters
    ----------
    lamination: LamSlot
        Lamintation with slot to draw
    save_path: str
        Path to save the msh result file
    sym : int
        Number of symmetry to apply
    mesh_size : float
        Size of the mesh [m]
    user_mesh_dict : dict
        To enforce the number of elements on the lines
    is_rect : bool
        To use rectangular elements
    Nlayer : int
        Number of mesh layer on Z axis
    display : bool
        To display gmsh logs
    Returns
    -------
    None
    """
    # The defaut symmetry is Zs => We draw only one tooth
    if sym == -1:
        tooth_surf = lamination.slot.get_surface_tooth()
        Zs = lamination.get_Zs()
    else:
        tooth_surf = lamination.build_geometry(sym=sym)[0]
        Zs = sym

    # For readibility
    model = gmsh.model
    factory = model.geo
    L = lamination.L1  # Lamination length

    # Start a new model
    gmsh.initialize(sys.argv)

    gmsh.option.setNumber("General.Terminal", int(display))

    gmsh.option.setNumber("Geometry.CopyMeshingMethod", 1)
    model.add("Pyleecan")

    # Create all the points of the tooth
    NPoint = 0  # Number of point created
    for line in tooth_surf.get_lines():
        Z = line.get_begin()
        NPoint += 1
        factory.addPoint(Z.real, Z.imag, -L / 2, mesh_size, NPoint)

    # Draw all the lines of the tooth
    NLine = 0  # Number of line created
    for line in tooth_surf.get_lines():
        NLine += 1
        if NLine == len(tooth_surf.get_lines()):
            if isinstance(line, Arc):
                Zc = line.get_center()
                NPoint += 1
                factory.addPoint(Zc.real, Zc.imag, -L / 2, mesh_size, NPoint)
                factory.addCircleArc(NLine, NPoint - 1, 1, NLine)
            else:
                factory.addLine(NLine, 1, NLine)
        else:
            if isinstance(line, Arc):
                Zc = line.get_center()
                NPoint += 1
                factory.addPoint(Zc.real, Zc.imag, -L / 2, mesh_size, NPoint)
                factory.addCircleArc(NLine, NPoint, NLine + 1, NLine)
            else:
                factory.addLine(NLine, NLine + 1, NLine)

    # Create the Tooth surface
    gmsh.model.geo.addCurveLoop(list(range(1, NLine + 1)), 1)
    gmsh.model.geo.addPlaneSurface([1], 1)
    gmsh.model.addPhysicalGroup(2, [1], 1)
    gmsh.model.setPhysicalName(2, 1, "Tooth")

    # convert triangle mesh to rectangle mesh
    if is_rect:
        factory.mesh.setRecombine(2, 1)

    # Change the mesh size for each line
    if user_mesh_dict is not None:
        # Compute basic mesh_dict
        mesh_dict = tooth_surf.comp_mesh_dict(element_size=mesh_size, label="Tooth")
        # Overwrite basic mesh dict with user one
        mesh_dict.update(user_mesh_dict)
        # Apply the number of element on each line of the surface
        for ii, line in enumerate(tooth_surf.get_lines()):
            factory.mesh.setTransfiniteCurve(
                ii + 1, mesh_dict[line.label] + 1, "Progression"
            )

    # Copy/Rotate all the tooth to get the 2D lamination
    surf_list = [1]
    for ii in range(Zs):
        ov = factory.copy([(2, 1)])
        factory.rotate(ov, 0, 0, -L / 2, 0, 0, 1, (ii + 1) * 2 * pi / Zs)
        surf_list.append(ov[0][1])
    gmsh.model.addPhysicalGroup(2, surf_list, 2)
    gmsh.model.setPhysicalName(2, 2, "Lamination")

    # Extrude the lamination
    for surf in surf_list:
        if is_rect == True:
            ov = factory.extrude(
                [(2, surf)], 0, 0, L, numElements=[Nlayer], recombine=True
            )
        else:
            ov = factory.extrude(
                [(2, surf)], 0, 0, L, numElements=[Nlayer], recombine=False
            )
    model.addPhysicalGroup(3, list(range(1, Zs + 1)), 1)
    if lamination.is_stator:
        model.setPhysicalName(3, 1, "stator")
    else:
        model.setPhysicalName(3, 1, "rotor")

    # Generate the 3D mesh
    factory.synchronize()
    gmsh.model.mesh.generate(3)

    # Save and close
    gmsh.write(save_path)
    gmsh.finalize()



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/Simu1.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/Simu1
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Simulation import Simulation

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.Simu1.run_single import run_single
except ImportError as error:
    run_single = error


from ._check import InitUnKnowClassError
from .Electrical import Electrical
from .Magnetics import Magnetics
from .Structural import Structural
from .Force import Force
from .Loss import Loss
from .Machine import Machine
from .Input import Input
from .VarSimu import VarSimu
from .Post import Post


class Simu1(Simulation):
    """Five sequential weak coupling multi physics simulation"""

    VERSION = 1

    # cf Methods.Simulation.Simu1.run_single
    if isinstance(run_single, ImportError):
        run_single = property(
            fget=lambda x: raise_(
                ImportError("Can't use Simu1 method run_single: " + str(run_single))
            )
        )
    else:
        run_single = run_single
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        elec=None,
        mag=None,
        struct=None,
        force=None,
        loss=None,
        name="",
        desc="",
        machine=-1,
        input=-1,
        logger_name="Pyleecan.Simulation",
        var_simu=None,
        postproc_list=-1,
        index=None,
        path_result=None,
        layer=None,
        layer_log_warn=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "elec" in list(init_dict.keys()):
                elec = init_dict["elec"]
            if "mag" in list(init_dict.keys()):
                mag = init_dict["mag"]
            if "struct" in list(init_dict.keys()):
                struct = init_dict["struct"]
            if "force" in list(init_dict.keys()):
                force = init_dict["force"]
            if "loss" in list(init_dict.keys()):
                loss = init_dict["loss"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "machine" in list(init_dict.keys()):
                machine = init_dict["machine"]
            if "input" in list(init_dict.keys()):
                input = init_dict["input"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "var_simu" in list(init_dict.keys()):
                var_simu = init_dict["var_simu"]
            if "postproc_list" in list(init_dict.keys()):
                postproc_list = init_dict["postproc_list"]
            if "index" in list(init_dict.keys()):
                index = init_dict["index"]
            if "path_result" in list(init_dict.keys()):
                path_result = init_dict["path_result"]
            if "layer" in list(init_dict.keys()):
                layer = init_dict["layer"]
            if "layer_log_warn" in list(init_dict.keys()):
                layer_log_warn = init_dict["layer_log_warn"]
        # Set the properties (value check and convertion are done in setter)
        self.elec = elec
        self.mag = mag
        self.struct = struct
        self.force = force
        self.loss = loss
        # Call Simulation init
        super(Simu1, self).__init__(
            name=name,
            desc=desc,
            machine=machine,
            input=input,
            logger_name=logger_name,
            var_simu=var_simu,
            postproc_list=postproc_list,
            index=index,
            path_result=path_result,
            layer=layer,
            layer_log_warn=layer_log_warn,
        )
        # The class is frozen (in Simulation init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Simu1_str = ""
        # Get the properties inherited from Simulation
        Simu1_str += super(Simu1, self).__str__()
        if self.elec is not None:
            tmp = self.elec.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simu1_str += "elec = " + tmp
        else:
            Simu1_str += "elec = None" + linesep + linesep
        if self.mag is not None:
            tmp = self.mag.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simu1_str += "mag = " + tmp
        else:
            Simu1_str += "mag = None" + linesep + linesep
        if self.struct is not None:
            tmp = self.struct.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simu1_str += "struct = " + tmp
        else:
            Simu1_str += "struct = None" + linesep + linesep
        if self.force is not None:
            tmp = self.force.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simu1_str += "force = " + tmp
        else:
            Simu1_str += "force = None" + linesep + linesep
        if self.loss is not None:
            tmp = self.loss.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simu1_str += "loss = " + tmp
        else:
            Simu1_str += "loss = None" + linesep + linesep
        return Simu1_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Simulation
        if not super(Simu1, self).__eq__(other):
            return False
        if other.elec != self.elec:
            return False
        if other.mag != self.mag:
            return False
        if other.struct != self.struct:
            return False
        if other.force != self.force:
            return False
        if other.loss != self.loss:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Simulation
        diff_list.extend(super(Simu1, self).compare(other, name=name))
        if (other.elec is None and self.elec is not None) or (
            other.elec is not None and self.elec is None
        ):
            diff_list.append(name + ".elec None mismatch")
        elif self.elec is not None:
            diff_list.extend(self.elec.compare(other.elec, name=name + ".elec"))
        if (other.mag is None and self.mag is not None) or (
            other.mag is not None and self.mag is None
        ):
            diff_list.append(name + ".mag None mismatch")
        elif self.mag is not None:
            diff_list.extend(self.mag.compare(other.mag, name=name + ".mag"))
        if (other.struct is None and self.struct is not None) or (
            other.struct is not None and self.struct is None
        ):
            diff_list.append(name + ".struct None mismatch")
        elif self.struct is not None:
            diff_list.extend(self.struct.compare(other.struct, name=name + ".struct"))
        if (other.force is None and self.force is not None) or (
            other.force is not None and self.force is None
        ):
            diff_list.append(name + ".force None mismatch")
        elif self.force is not None:
            diff_list.extend(self.force.compare(other.force, name=name + ".force"))
        if (other.loss is None and self.loss is not None) or (
            other.loss is not None and self.loss is None
        ):
            diff_list.append(name + ".loss None mismatch")
        elif self.loss is not None:
            diff_list.extend(self.loss.compare(other.loss, name=name + ".loss"))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Simulation
        S += super(Simu1, self).__sizeof__()
        S += getsizeof(self.elec)
        S += getsizeof(self.mag)
        S += getsizeof(self.struct)
        S += getsizeof(self.force)
        S += getsizeof(self.loss)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Simulation
        Simu1_dict = super(Simu1, self).as_dict(**kwargs)
        if self.elec is None:
            Simu1_dict["elec"] = None
        else:
            Simu1_dict["elec"] = self.elec.as_dict(**kwargs)
        if self.mag is None:
            Simu1_dict["mag"] = None
        else:
            Simu1_dict["mag"] = self.mag.as_dict(**kwargs)
        if self.struct is None:
            Simu1_dict["struct"] = None
        else:
            Simu1_dict["struct"] = self.struct.as_dict(**kwargs)
        if self.force is None:
            Simu1_dict["force"] = None
        else:
            Simu1_dict["force"] = self.force.as_dict(**kwargs)
        if self.loss is None:
            Simu1_dict["loss"] = None
        else:
            Simu1_dict["loss"] = self.loss.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        Simu1_dict["__class__"] = "Simu1"
        return Simu1_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.elec is not None:
            self.elec._set_None()
        if self.mag is not None:
            self.mag._set_None()
        if self.struct is not None:
            self.struct._set_None()
        if self.force is not None:
            self.force._set_None()
        if self.loss is not None:
            self.loss._set_None()
        # Set to None the properties inherited from Simulation
        super(Simu1, self)._set_None()

    def _get_elec(self):
        """getter of elec"""
        return self._elec

    def _set_elec(self, value):
        """setter of elec"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "elec")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Electrical()
        check_var("elec", value, "Electrical")
        self._elec = value

        if self._elec is not None:
            self._elec.parent = self

    elec = property(
        fget=_get_elec,
        fset=_set_elec,
        doc=u"""Electrical module

        :Type: Electrical
        """,
    )

    def _get_mag(self):
        """getter of mag"""
        return self._mag

    def _set_mag(self, value):
        """setter of mag"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "mag")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Magnetics()
        check_var("mag", value, "Magnetics")
        self._mag = value

        if self._mag is not None:
            self._mag.parent = self

    mag = property(
        fget=_get_mag,
        fset=_set_mag,
        doc=u"""Magnetic module

        :Type: Magnetics
        """,
    )

    def _get_struct(self):
        """getter of struct"""
        return self._struct

    def _set_struct(self, value):
        """setter of struct"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "struct"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Structural()
        check_var("struct", value, "Structural")
        self._struct = value

        if self._struct is not None:
            self._struct.parent = self

    struct = property(
        fget=_get_struct,
        fset=_set_struct,
        doc=u"""Structural module

        :Type: Structural
        """,
    )

    def _get_force(self):
        """getter of force"""
        return self._force

    def _set_force(self, value):
        """setter of force"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "force"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Force()
        check_var("force", value, "Force")
        self._force = value

        if self._force is not None:
            self._force.parent = self

    force = property(
        fget=_get_force,
        fset=_set_force,
        doc=u"""Force moduale

        :Type: Force
        """,
    )

    def _get_loss(self):
        """getter of loss"""
        return self._loss

    def _set_loss(self, value):
        """setter of loss"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "loss")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Loss()
        check_var("loss", value, "Loss")
        self._loss = value

        if self._loss is not None:
            self._loss.parent = self

    loss = property(
        fget=_get_loss,
        fset=_set_loss,
        doc=u"""Loss moduale

        :Type: Loss
        """,
    )



# -*- coding: utf-8 -*-

from numpy import zeros

from ....Methods.Machine.Winding import WindingError
from ....Functions.Winding.reverse_wind_mat import reverse_wind_mat
from ....Functions.Winding.shift_wind_mat import shift_wind_mat

# TODO: update docstring -> there should not be elementary circuits since
#       every bar could have its own unique current
def comp_connection_mat(self, Zs=None):
    """Compute the Winding Matrix (for winding type 10)
    type 10 : Squirrel cage Winding (elementary circuit loop involving bar n°1
    and bar n°Zr for alphar0_rad=0) (Nlay_rad=Nlay_tan=1)

    Parameters
    ----------
    self : Winding
        A: Winding object
    Zs : int
        Number of Slot (Integer >0)

    Returns
    -------
    numpy.ndarray
        Winding Matrix (Nlay_rad, Nlay_tan, Zs, qs)

    Raises
    ------
    WindingT10DefQsError
        qs must be equal to Zs

    """
    if Zs is None:
        if self.parent is None:
            raise WindingError(
                "ERROR: The Winding object must be in a Lamination object."
            )

        if self.parent.slot is None:
            raise WindingError(
                "ERROR: The Winding object must be in a Lamination object with Slot."
            )

        Zs = self.parent.slot.Zs

    assert Zs > 0, "Zs must be >0"
    assert Zs % 1 == 0, "Zs must be an integer"

    if self.qs != Zs:
        raise WindingT10DefQsError(
            "wrong winding definition, you must have " "qs = Zs !"
        )

    wind_mat = zeros((1, 1, Zs, self.qs))

    for ii in range(Zs):
        # phase n°ii
        # wind_mat[0, 0, ii, ii] = -1 # there are only positive 'windings' for now
        # wind_mat[0, 0, (ii - 1) % Zs, ii] = 1
        wind_mat[0, 0, ii, ii] = 1

    # Set default values
    if self.is_reverse_wind is None:
        self.is_reverse_wind = False
    if self.Nslot_shift_wind is None:
        self.Nslot_shift_wind = 0
    # Apply the transformations
    if self.is_reverse_wind:
        wind_mat = reverse_wind_mat(wind_mat)
    if self.Nslot_shift_wind > 0:
        wind_mat = shift_wind_mat(wind_mat, self.Nslot_shift_wind)

    return wind_mat


class WindingT10DefQsError(WindingError):
    """ """

    pass



# -*- coding: utf-8 -*-

from numpy import max as np_max, min as np_min


def plot(self, i):
    """Return the pyvista mesh object.

    Parameters
    ----------
    self : Mode
        a Mode object
    i : int
        index of the mode to plot

    Returns
    -------
    mesh : pyvista.core.pointset.StructuredGrid
        a pyvista StructuredGrid object
    """

    radial_shape = self.get_shape_pol()[:, 0]
    clim = [np_min(radial_shape), np_max(radial_shape)]
    self.parent.mesh.plot_contour(
        radial_shape, field_name="Radial displacement", clim=clim
    )



# -*- coding: utf-8 -*-


def set_m(self, value):
    """Convert the value the correct m unit (to set in the object in m)

    Parameters
    ----------
    self : Unit
        A Unit object
    value : float
        Value to convert

    Returns
    -------
    value: float
        value converted in [m]
    """
    if self.unit_m == 1:  # Convert to mm
        return value / 1000
    else:
        return value



def comp_length_endwinding(self):
    """Compute the Winding overhang length on one side for a half-turn[m]

    Parameters
    ----------
    self: Winding
        A Winding object
    Returns
    -------
    Lewout : float
        End-winding length on one side for a half-turn [m].
    """

    # TODO: compute it
    return self.Lewout



from numpy import arcsin, exp


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotM11
        A SlotM11 object

    Returns
    -------
    point_dict: dict
        A dict of the slot coordinates
    """

    Rbo = self.get_Rbo()

    Z1 = Rbo * exp(-1j * self.W0 / 2)

    if self.is_outwards():
        Z2 = (Rbo + self.H0) * exp(-1j * self.W0 / 2)
        ZM1 = (Rbo + self.H0) * exp(-1j * self.Wmag / 2)
        ZM2 = (Rbo + self.H0 - self.Hmag) * exp(-1j * self.Wmag / 2)
    else:  # inward slot
        Z2 = (Rbo - self.H0) * exp(-1j * self.W0 / 2)
        ZM1 = (Rbo - self.H0) * exp(-1j * self.Wmag / 2)
        ZM2 = (Rbo - self.H0 + self.Hmag) * exp(-1j * self.Wmag / 2)

    point_dict = dict()
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["ZM1"] = ZM1
    point_dict["ZM2"] = ZM2
    # symetry

    point_dict["Z3"] = Z2.conjugate()
    point_dict["Z4"] = Z1.conjugate()
    point_dict["ZM3"] = ZM2.conjugate()
    point_dict["ZM4"] = ZM1.conjugate()

    return point_dict



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Slot/Hole.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Slot/Hole
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Slot.Hole.comp_height import comp_height
except ImportError as error:
    comp_height = error

try:
    from ..Methods.Slot.Hole.comp_magnetization_dict import comp_magnetization_dict
except ImportError as error:
    comp_magnetization_dict = error

try:
    from ..Methods.Slot.Hole.comp_radius import comp_radius
except ImportError as error:
    comp_radius = error

try:
    from ..Methods.Slot.Hole.comp_surface import comp_surface
except ImportError as error:
    comp_surface = error

try:
    from ..Methods.Slot.Hole.convert_to_UD import convert_to_UD
except ImportError as error:
    convert_to_UD = error

try:
    from ..Methods.Slot.Hole.get_is_stator import get_is_stator
except ImportError as error:
    get_is_stator = error

try:
    from ..Methods.Slot.Hole.get_magnet_by_id import get_magnet_by_id
except ImportError as error:
    get_magnet_by_id = error

try:
    from ..Methods.Slot.Hole.get_magnet_dict import get_magnet_dict
except ImportError as error:
    get_magnet_dict = error

try:
    from ..Methods.Slot.Hole.get_Rbo import get_Rbo
except ImportError as error:
    get_Rbo = error

try:
    from ..Methods.Slot.Hole.get_Rext import get_Rext
except ImportError as error:
    get_Rext = error

try:
    from ..Methods.Slot.Hole.has_magnet import has_magnet
except ImportError as error:
    has_magnet = error

try:
    from ..Methods.Slot.Hole.plot import plot
except ImportError as error:
    plot = error

try:
    from ..Methods.Slot.Hole.set_magnet_by_id import set_magnet_by_id
except ImportError as error:
    set_magnet_by_id = error


from ._check import InitUnKnowClassError
from .Material import Material


class Hole(FrozenClass):
    """Holes for lamination (abstract)"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Slot.Hole.comp_height
    if isinstance(comp_height, ImportError):
        comp_height = property(
            fget=lambda x: raise_(
                ImportError("Can't use Hole method comp_height: " + str(comp_height))
            )
        )
    else:
        comp_height = comp_height
    # cf Methods.Slot.Hole.comp_magnetization_dict
    if isinstance(comp_magnetization_dict, ImportError):
        comp_magnetization_dict = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Hole method comp_magnetization_dict: "
                    + str(comp_magnetization_dict)
                )
            )
        )
    else:
        comp_magnetization_dict = comp_magnetization_dict
    # cf Methods.Slot.Hole.comp_radius
    if isinstance(comp_radius, ImportError):
        comp_radius = property(
            fget=lambda x: raise_(
                ImportError("Can't use Hole method comp_radius: " + str(comp_radius))
            )
        )
    else:
        comp_radius = comp_radius
    # cf Methods.Slot.Hole.comp_surface
    if isinstance(comp_surface, ImportError):
        comp_surface = property(
            fget=lambda x: raise_(
                ImportError("Can't use Hole method comp_surface: " + str(comp_surface))
            )
        )
    else:
        comp_surface = comp_surface
    # cf Methods.Slot.Hole.convert_to_UD
    if isinstance(convert_to_UD, ImportError):
        convert_to_UD = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Hole method convert_to_UD: " + str(convert_to_UD)
                )
            )
        )
    else:
        convert_to_UD = convert_to_UD
    # cf Methods.Slot.Hole.get_is_stator
    if isinstance(get_is_stator, ImportError):
        get_is_stator = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Hole method get_is_stator: " + str(get_is_stator)
                )
            )
        )
    else:
        get_is_stator = get_is_stator
    # cf Methods.Slot.Hole.get_magnet_by_id
    if isinstance(get_magnet_by_id, ImportError):
        get_magnet_by_id = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Hole method get_magnet_by_id: " + str(get_magnet_by_id)
                )
            )
        )
    else:
        get_magnet_by_id = get_magnet_by_id
    # cf Methods.Slot.Hole.get_magnet_dict
    if isinstance(get_magnet_dict, ImportError):
        get_magnet_dict = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Hole method get_magnet_dict: " + str(get_magnet_dict)
                )
            )
        )
    else:
        get_magnet_dict = get_magnet_dict
    # cf Methods.Slot.Hole.get_Rbo
    if isinstance(get_Rbo, ImportError):
        get_Rbo = property(
            fget=lambda x: raise_(
                ImportError("Can't use Hole method get_Rbo: " + str(get_Rbo))
            )
        )
    else:
        get_Rbo = get_Rbo
    # cf Methods.Slot.Hole.get_Rext
    if isinstance(get_Rext, ImportError):
        get_Rext = property(
            fget=lambda x: raise_(
                ImportError("Can't use Hole method get_Rext: " + str(get_Rext))
            )
        )
    else:
        get_Rext = get_Rext
    # cf Methods.Slot.Hole.has_magnet
    if isinstance(has_magnet, ImportError):
        has_magnet = property(
            fget=lambda x: raise_(
                ImportError("Can't use Hole method has_magnet: " + str(has_magnet))
            )
        )
    else:
        has_magnet = has_magnet
    # cf Methods.Slot.Hole.plot
    if isinstance(plot, ImportError):
        plot = property(
            fget=lambda x: raise_(
                ImportError("Can't use Hole method plot: " + str(plot))
            )
        )
    else:
        plot = plot
    # cf Methods.Slot.Hole.set_magnet_by_id
    if isinstance(set_magnet_by_id, ImportError):
        set_magnet_by_id = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Hole method set_magnet_by_id: " + str(set_magnet_by_id)
                )
            )
        )
    else:
        set_magnet_by_id = set_magnet_by_id
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        Zh=36,
        mat_void=-1,
        magnetization_dict_offset=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Zh" in list(init_dict.keys()):
                Zh = init_dict["Zh"]
            if "mat_void" in list(init_dict.keys()):
                mat_void = init_dict["mat_void"]
            if "magnetization_dict_offset" in list(init_dict.keys()):
                magnetization_dict_offset = init_dict["magnetization_dict_offset"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.Zh = Zh
        self.mat_void = mat_void
        self.magnetization_dict_offset = magnetization_dict_offset

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Hole_str = ""
        if self.parent is None:
            Hole_str += "parent = None " + linesep
        else:
            Hole_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Hole_str += "Zh = " + str(self.Zh) + linesep
        if self.mat_void is not None:
            tmp = self.mat_void.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Hole_str += "mat_void = " + tmp
        else:
            Hole_str += "mat_void = None" + linesep + linesep
        Hole_str += (
            "magnetization_dict_offset = "
            + str(self.magnetization_dict_offset)
            + linesep
        )
        return Hole_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.Zh != self.Zh:
            return False
        if other.mat_void != self.mat_void:
            return False
        if other.magnetization_dict_offset != self.magnetization_dict_offset:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._Zh != self._Zh:
            diff_list.append(name + ".Zh")
        if (other.mat_void is None and self.mat_void is not None) or (
            other.mat_void is not None and self.mat_void is None
        ):
            diff_list.append(name + ".mat_void None mismatch")
        elif self.mat_void is not None:
            diff_list.extend(
                self.mat_void.compare(other.mat_void, name=name + ".mat_void")
            )
        if other._magnetization_dict_offset != self._magnetization_dict_offset:
            diff_list.append(name + ".magnetization_dict_offset")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.Zh)
        S += getsizeof(self.mat_void)
        if self.magnetization_dict_offset is not None:
            for key, value in self.magnetization_dict_offset.items():
                S += getsizeof(value) + getsizeof(key)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Hole_dict = dict()
        Hole_dict["Zh"] = self.Zh
        if self.mat_void is None:
            Hole_dict["mat_void"] = None
        else:
            Hole_dict["mat_void"] = self.mat_void.as_dict(**kwargs)
        Hole_dict["magnetization_dict_offset"] = (
            self.magnetization_dict_offset.copy()
            if self.magnetization_dict_offset is not None
            else None
        )
        # The class name is added to the dict for deserialisation purpose
        Hole_dict["__class__"] = "Hole"
        return Hole_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Zh = None
        if self.mat_void is not None:
            self.mat_void._set_None()
        self.magnetization_dict_offset = None

    def _get_Zh(self):
        """getter of Zh"""
        return self._Zh

    def _set_Zh(self, value):
        """setter of Zh"""
        check_var("Zh", value, "int", Vmin=0, Vmax=1000)
        self._Zh = value

    Zh = property(
        fget=_get_Zh,
        fset=_set_Zh,
        doc=u"""Number of Hole around the circumference

        :Type: int
        :min: 0
        :max: 1000
        """,
    )

    def _get_mat_void(self):
        """getter of mat_void"""
        return self._mat_void

    def _set_mat_void(self, value):
        """setter of mat_void"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "mat_void"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Material()
        check_var("mat_void", value, "Material")
        self._mat_void = value

        if self._mat_void is not None:
            self._mat_void.parent = self

    mat_void = property(
        fget=_get_mat_void,
        fset=_set_mat_void,
        doc=u"""Material of the void part of the hole (Air in general)

        :Type: Material
        """,
    )

    def _get_magnetization_dict_offset(self):
        """getter of magnetization_dict_offset"""
        return self._magnetization_dict_offset

    def _set_magnetization_dict_offset(self, value):
        """setter of magnetization_dict_offset"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("magnetization_dict_offset", value, "dict")
        self._magnetization_dict_offset = value

    magnetization_dict_offset = property(
        fget=_get_magnetization_dict_offset,
        fset=_set_magnetization_dict_offset,
        doc=u"""Dictionary add an offset to the magnetization direction of the magnets (key=magnet_X, value=angle[rad])

        :Type: dict
        """,
    )
